package com.uoc.taller.adapter.openalpr;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.inject.Inject;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Optional;
import java.util.logging.Level;
import java.util.logging.Logger;

class JpegCompressor {
    private static float QUALITY = 0.5f;

    @Inject
    private Logger logger;

    Optional<InputStream> compress(InputStream inputStream) {

        try (ByteArrayOutputStream os = new ByteArrayOutputStream(); ImageOutputStream imageOutputStream = ImageIO.createImageOutputStream(os)) {
            BufferedImage image = ImageIO.read(inputStream);
            ImageWriter writer = getImageWriter(imageOutputStream);
            writer.write(null, new IIOImage(image, null, null), createImageWriteParams(writer));

            //In case you want to see modified image
            //writer.setOutput(ImageIO.createImageOutputStream(new FileOutputStream(new File("image_modified.jpg"))));
            //writer.write(null, new IIOImage(image, null, null), createImageWriteParams(writer));
            byte[] result = os.toByteArray();
            logger.log(Level.SEVERE, "After compress: " + result.length);
            return Optional.of(new ByteArrayInputStream(result));
        } catch (IOException e) {
            logger.log(Level.SEVERE, "Unable to compress image.", e);
            return Optional.empty();
        }
    }

    private ImageWriter getImageWriter(ImageOutputStream imageOutputStream) throws IOException {
        Iterator<ImageWriter> writers = ImageIO.getImageWritersByFormatName("jpg");

        if (!writers.hasNext())
            throw new IOException("No writers found");

        ImageWriter imageWriter = writers.next();
        imageWriter.setOutput(imageOutputStream);
        return imageWriter;
    }

    private ImageWriteParam createImageWriteParams(ImageWriter writer) {
        ImageWriteParam param = writer.getDefaultWriteParam();

        param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
        param.setCompressionQuality(QUALITY);
        return param;
    }
}
