package com.uoc.taller.adapter.openalpr;

import com.uoc.taller.domain.port.OCRSolution;

import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.core.Response;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Base64;
import java.util.Optional;

public class OpenALPRClient implements OCRSolution {

    private static final String COUNTRY = "country";
    private static final String SECRET_KEY = "secret_key";
    private static final String MEDIA_TYPE = "image/jpeg";
    private static final String RECOGNIZE_BYTES = "recognize_bytes";
    private static final String EMPTY_STRING = "";
    private static final int LIMIT_SIZE = 2048000;

    private final String url;
    private final String secretKey;
    private final Client client;
    private final String country = "eu";
    private final JpegCompressor jpegCompressor;

    @Inject
    public OpenALPRClient(JpegCompressor jpegCompressor) {
        this(jpegCompressor, "https://api.openalpr.com/v2", "sk_ce69d629e86f393a4dc0b056");
    }


    public OpenALPRClient(JpegCompressor jpegCompressor, URL url, String secretKey) {
        this(jpegCompressor, url.toString(), secretKey);
    }

    public OpenALPRClient(JpegCompressor jpegCompressor, String url, String secretKey) {
        this.secretKey = secretKey;
        this.url = url;
        this.client = ClientBuilder.newClient();
        this.jpegCompressor = jpegCompressor;
    }


    @Override
    public String checkPlate(File file, String country) throws IOException {
        return checkPlate(new FileInputStream(file), country);
    }

    @Override
    public String checkPlate(InputStream inputStream, String country) throws IOException {
        //byte[] input = toByteArray(inputStream);
        /*if (input.length > LIMIT_SIZE) {
            Optional<InputStream> image = jpegCompressor.compress(new ByteArrayInputStream(input));
            if (image.isPresent()) {
                return sendRequest(image.get(), RECOGNIZE_BYTES, country);
            }
        }*/
        return sendRequest(inputStream, RECOGNIZE_BYTES, country);
    }

    private String sendRequest(InputStream inputStream, String path, String country) throws IOException {
        return processResponse(buildRequest(inputStream, path, country));
    }

    private Response buildRequest(InputStream inputStream, String path, String country) throws IOException {
        return client.target(url)
                .path(path)
                .queryParam(COUNTRY, country != null ? country : this.country)
                .queryParam(SECRET_KEY, secretKey)
                .request()
                .post(Entity.entity(encodeInputStream(inputStream), MEDIA_TYPE));
    }

    private byte[] encodeInputStream(InputStream inputStream) throws IOException {
        return Base64.getEncoder().encode(toByteArray(inputStream));
    }

    private String processResponse(Response response) {
        return Optional.of(response)
                .filter(resp -> resp.getStatus() == 200)
                .map(resp -> Json.createReader((InputStream) response.getEntity())
                        .readObject())
                .map(json -> json.getJsonArray("results"))
                .filter(jsonValues -> jsonValues.size() > 0)
                .map(jsonValues -> jsonValues.getJsonObject(0))
                .map(jsonObject -> jsonObject.getString("plate"))
                .orElse(EMPTY_STRING);
    }

    private byte[] toByteArray(InputStream in) throws IOException {
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len;
        while ((len = in.read(buffer)) != -1) {
            os.write(buffer, 0, len);
        }
        return os.toByteArray();
    }

}
