package com.uoc.taller.adapter.reports;

import com.uoc.taller.adapter.reports.flyingsaucer.FlyingSaucerPdfGenerator;
import com.uoc.taller.adapter.reports.thymeleaf.HtmlThymeleafTemplateGenerator;
import com.uoc.taller.domain.port.ReportGenerator;
import org.thymeleaf.context.Context;

import java.io.File;
import java.util.Optional;

public abstract class AbstractReportGenerator<T> implements ReportGenerator<T> {
    protected final HtmlThymeleafTemplateGenerator htmlThymeleafTemplateGenerator;
    protected final FlyingSaucerPdfGenerator flyingSaucerPdfGenerator;


    AbstractReportGenerator(HtmlThymeleafTemplateGenerator htmlThymeleafTemplateGenerator, FlyingSaucerPdfGenerator flyingSaucerPdfGenerator) {
        this.htmlThymeleafTemplateGenerator = htmlThymeleafTemplateGenerator;
        this.flyingSaucerPdfGenerator = flyingSaucerPdfGenerator;
    }

    @Override
    public Optional<File> generatePDF(T data, String fileName) {
        return flyingSaucerPdfGenerator.processContent(fileName, generateHtml(data));
    }

    @Override
    public String generateHtml(T data) {
        return htmlThymeleafTemplateGenerator.processTemplate(getTemplateName(), createContext(data));
    }

    @Override
    public Optional<File> generateHTMLFile(String data, String fileName) {
        return htmlThymeleafTemplateGenerator.processContent(fileName, data);
    }

    protected abstract Context createContext(T contextFrom);
    protected abstract String getTemplateName();
}
