package com.uoc.taller.adapter.reports;

import com.uoc.taller.adapter.reports.flyingsaucer.FlyingSaucerPdfGenerator;
import com.uoc.taller.adapter.reports.labels.LabelManager;
import com.uoc.taller.adapter.reports.thymeleaf.HtmlThymeleafTemplateGenerator;
import com.uoc.taller.domain.entity.GuardTicket;
import org.thymeleaf.context.Context;

import javax.inject.Inject;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

public class GuardTicketReportGenerator extends AbstractReportGenerator<GuardTicket> {

    private static final String TEMPLATE_NAME = "guardTicket.html";
    private final LabelManager labelManager;

    @Inject
    public GuardTicketReportGenerator(HtmlThymeleafTemplateGenerator htmlThymeleafTemplateGenerator, FlyingSaucerPdfGenerator flyingSaucerPdfGenerator, LabelManager labelManager) {
        super(htmlThymeleafTemplateGenerator, flyingSaucerPdfGenerator);
        this.labelManager = labelManager;
    }

    @Override
    protected Context createContext(GuardTicket guardTicket) {
        DateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy");
        Context context = new Context();
        context.setVariables(labelManager.getLabelsByCategory("guardTicket"));
        context.setVariables(labelManager.getLabelsByCategory("vehicle"));
        context.setVariables(labelManager.getLabelsByCategory("commonData"));

        context.setVariable("id", guardTicket.getId());
        context.setVariable("dateOfIssue", dateFormat.format(guardTicket.getDateOfIssue()));
        context.setVariable("expectedDate", dateFormat.format(guardTicket.getExpectedDate()));
        context.setVariable("tallerFiscalName", guardTicket.getTaller().getTaxName());
        context.setVariable("tallerNif", guardTicket.getTaller().getNif());
        context.setVariable("tallerAddress", guardTicket.getTaller().getAddress());
        context.setVariable("tallerPostalCode", guardTicket.getTaller().getPostalCode());
        context.setVariable("tallerCity", guardTicket.getTaller().getCity());
        context.setVariable("tallerProvince", guardTicket.getTaller().getProvince());
        context.setVariable("tallerCountry", guardTicket.getTaller().getCountry());
        context.setVariable("tallerTelephone", guardTicket.getTaller().getTelephone());
        context.setVariable("tallerEmail", guardTicket.getTaller().getEmail());
        context.setVariable("observations", guardTicket.getObservations());
        context.setVariable("userNif", guardTicket.getUserNif());
        context.setVariable("userTelephone", guardTicket.getUserTelephone());
        context.setVariable("userMail", guardTicket.getUserMail());
        context.setVariable("userFiscalName", guardTicket.getUserFiscalName());
        context.setVariable("userAddress", guardTicket.getUserAddress());
        context.setVariable("userPostalCode", guardTicket.getUserPostalCode());
        context.setVariable("userCity", guardTicket.getUserCity());
        context.setVariable("userProvince", guardTicket.getUserProvince());
        context.setVariable("userCountry", guardTicket.getUserCountry());

        context.setVariable("numberKilometers", guardTicket.getNumberKilometers());

        context.setVariable("vehiclePlate", guardTicket.getVehicle().getPlate());
        context.setVariable("vehicleBrand", guardTicket.getVehicle().getBrand());
        context.setVariable("vehicleModel", guardTicket.getVehicle().getModel());
        context.setVariable("vehicleChassisNumber", guardTicket.getVehicle().getChassisNumber());

        return context;
    }

    @Override
    protected String getTemplateName() {
        return TEMPLATE_NAME;
    }
}
