package com.uoc.taller.adapter.reports;

import com.uoc.taller.adapter.reports.flyingsaucer.FlyingSaucerPdfGenerator;
import com.uoc.taller.adapter.reports.labels.LabelManager;
import com.uoc.taller.adapter.reports.thymeleaf.HtmlThymeleafTemplateGenerator;
import com.uoc.taller.domain.entity.DocumentItem;
import com.uoc.taller.domain.entity.Invoice;
import org.thymeleaf.context.Context;

import javax.inject.Inject;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;


public class InvoiceReportGenerator extends AbstractReportGenerator<Invoice> {

    private static final String TEMPLATE_NAME = "invoice.html";
    private final LabelManager labelManager;
    private float billTotal;
    private DecimalFormat df;

    @Inject
    public InvoiceReportGenerator(HtmlThymeleafTemplateGenerator htmlThymeleafTemplateGenerator, FlyingSaucerPdfGenerator flyingSaucerPdfGenerator, LabelManager labelManager) {
        super(htmlThymeleafTemplateGenerator, flyingSaucerPdfGenerator);
        this.labelManager = labelManager;
        this.df = new DecimalFormat();
        df.setMaximumFractionDigits(2);
    }


    @Override
    protected Context createContext(Invoice invoice) {
        DateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy");
        Context context = new Context();
        context.setVariables(labelManager.getLabelsByCategory("products"));
        context.setVariables(labelManager.getLabelsByCategory("invoice"));
        context.setVariables(labelManager.getLabelsByCategory("vehicle"));
        context.setVariables(labelManager.getLabelsByCategory("commonData"));
        context.setVariable("id", invoice.getId());
        context.setVariable("dateOfIssue", dateFormat.format(invoice.getDateOfIssue()));
//        context.setVariable("expectedDate", dateFormat.format(invoice.getExpectedDate()));
        context.setVariable("tallerFiscalName", invoice.getTaller().getTaxName());
        context.setVariable("tallerNif", invoice.getTaller().getNif());
        context.setVariable("tallerAddress", invoice.getTaller().getAddress());
        context.setVariable("tallerPostalCode", invoice.getTaller().getPostalCode());
        context.setVariable("tallerCity", invoice.getTaller().getCity());
        context.setVariable("tallerProvince", invoice.getTaller().getProvince());
        context.setVariable("tallerCountry", invoice.getTaller().getCountry());
        context.setVariable("tallerTelephone", invoice.getTaller().getTelephone());
        context.setVariable("tallerEmail", invoice.getTaller().getEmail());
        context.setVariable("observations", invoice.getObservations());
        context.setVariable("userNif", invoice.getUserNif());
        context.setVariable("userFiscalName", invoice.getUserFiscalName());
        context.setVariable("userTelephone", invoice.getUserTelephone());
        context.setVariable("userMail", invoice.getUserMail());
        context.setVariable("userAddress", invoice.getUserAddress());
        context.setVariable("userPostalCode", invoice.getUserPostalCode());
        context.setVariable("userCity", invoice.getUserCity());
        context.setVariable("userProvince", invoice.getUserProvince());
        context.setVariable("userCountry", invoice.getUserCountry());
        context.setVariable("paymentMethod", invoice.getPaymentMethod().getPaymentMethodName());

        context.setVariable("productsRowSet", getProductRowSet(invoice));
        context.setVariable("billTotal",df.format(billTotal));

        context.setVariable("numberKilometers", invoice.getNumberKilometers());

        context.setVariable("vehiclePlate", invoice.getVehicle().getPlate());
        context.setVariable("vehicleBrand", invoice.getVehicle().getBrand());
        context.setVariable("vehicleModel", invoice.getVehicle().getModel());
        context.setVariable("vehicleChassisNumber", invoice.getVehicle().getChassisNumber());
        return context;
    }

    private String getProductRowSet(Invoice invoice) {
        billTotal = 0f;
        StringBuilder ret = new StringBuilder();
        for (DocumentItem doc : invoice.getDocumentItems()) {
            ret.append("<tr><td>")
                    .append(doc.getProduct().getCode())
                    .append("</td><td>")
                    .append(doc.getProduct().getDescription())
                    .append("</td><td class='txtAlignCenter'>")
                    .append(doc.getQuantity())
                    .append("</td><td class='txtAlignRight'>")
                    .append(doc.getProductPrice())
                    .append("</td><td class='txtAlignCenter'>")
                    .append(doc.getTaxPercentage())
                    .append("</td><td class='txtAlignCenter'>")
                    .append(doc.getDiscountPercentage())
                    .append("</td><td class='txtAlignRight'>")
                    .append(df.format(getTotalDocumentItem(doc)))
                    .append("</td></tr>");
        }
        return ret.toString();
    }

    private float getTotalDocumentItem(DocumentItem doc) {
        float ret = doc.getQuantity() * (doc.getProductPrice() * ((100 + doc.getTaxPercentage()) / 100) * (100 - doc.getDiscountPercentage()) / 100);
        billTotal += ret;
        return ret;
    }

    @Override
    protected String getTemplateName() {
        return TEMPLATE_NAME;
    }
}
