package com.uoc.taller.adapter.reports;

import com.uoc.taller.adapter.reports.flyingsaucer.FlyingSaucerPdfGenerator;
import com.uoc.taller.adapter.reports.labels.LabelManager;
import com.uoc.taller.adapter.reports.thymeleaf.HtmlThymeleafTemplateGenerator;
import com.uoc.taller.domain.entity.DocumentItem;
import com.uoc.taller.domain.entity.WorkOrder;
import org.thymeleaf.context.Context;

import javax.inject.Inject;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;

public class WorkOrderReportGenerator extends AbstractReportGenerator<WorkOrder> {

    private static final String TEMPLATE_NAME = "workOrder.html";
    private final LabelManager labelManager;
    private float billTotal;
    private DecimalFormat df;

    @Inject
    WorkOrderReportGenerator(HtmlThymeleafTemplateGenerator htmlThymeleafTemplateGenerator, FlyingSaucerPdfGenerator flyingSaucerPdfGenerator, LabelManager labelManager) {
        super(htmlThymeleafTemplateGenerator, flyingSaucerPdfGenerator);
        this.labelManager = labelManager;
        this.df = new DecimalFormat();
        df.setMaximumFractionDigits(2);
    }

    @Override
    protected Context createContext(WorkOrder workOrder) {
        DateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy");
        Context context = new Context();
        context.setVariables(labelManager.getLabelsByCategory("products"));
        context.setVariables(labelManager.getLabelsByCategory("workOrder"));
        context.setVariables(labelManager.getLabelsByCategory("vehicle"));
        context.setVariables(labelManager.getLabelsByCategory("commonData"));
        context.setVariable("id", workOrder.getId());
        context.setVariable("dateOfIssue", dateFormat.format(workOrder.getDateOfIssue()));
        context.setVariable("expectedDate", dateFormat.format(workOrder.getExpectedDate()));
        context.setVariable("tallerFiscalName", workOrder.getTaller().getTaxName());
        context.setVariable("tallerNif", workOrder.getTaller().getNif());
        context.setVariable("tallerAddress", workOrder.getTaller().getAddress());
        context.setVariable("tallerPostalCode", workOrder.getTaller().getPostalCode());
        context.setVariable("tallerCity", workOrder.getTaller().getCity());
        context.setVariable("tallerProvince", workOrder.getTaller().getProvince());
        context.setVariable("tallerCountry", workOrder.getTaller().getCountry());
        context.setVariable("tallerTelephone", workOrder.getTaller().getTelephone());
        context.setVariable("tallerEmail", workOrder.getTaller().getEmail());
        context.setVariable("observations", workOrder.getObservations());
        context.setVariable("userNif", workOrder.getUserNif());
        context.setVariable("userFiscalName", workOrder.getUserFiscalName());
        context.setVariable("userTelephone", workOrder.getUserTelephone());
        context.setVariable("userMail", workOrder.getUserMail());
        context.setVariable("userAddress", workOrder.getUserAddress());
        context.setVariable("userPostalCode", workOrder.getUserPostalCode());
        context.setVariable("userCity", workOrder.getUserCity());
        context.setVariable("userProvince", workOrder.getUserProvince());
        context.setVariable("userCountry", workOrder.getUserCountry());

        context.setVariable("productsRowSet", getProductRowSet(workOrder));
        context.setVariable("billTotal",df.format(billTotal));

        context.setVariable("numberKilometers", workOrder.getNumberKilometers());

        context.setVariable("vehiclePlate", workOrder.getVehicle().getPlate());
        context.setVariable("vehicleBrand", workOrder.getVehicle().getBrand());
        context.setVariable("vehicleModel", workOrder.getVehicle().getModel());
        context.setVariable("vehicleChassisNumber", workOrder.getVehicle().getChassisNumber());
        return context;
    }

    private String getProductRowSet(WorkOrder workOrder) {
        billTotal = 0f; //Reset value
        StringBuilder ret = new StringBuilder();
        for (DocumentItem doc : workOrder.getDocumentItems()) {
            ret.append("<tr><td>")
                    .append(doc.getProduct().getCode())
                    .append("</td><td>")
                    .append(doc.getProduct().getDescription())
                    .append("</td><td class='txtAlignCenter'>")
                    .append(doc.getQuantity())
                    .append("</td><td class='txtAlignRight'>")
                    .append(doc.getProductPrice())
                    .append("</td><td class='txtAlignCenter'>")
                    .append(doc.getTaxPercentage())
                    .append("</td><td class='txtAlignCenter'>")
                    .append(doc.getDiscountPercentage())
                    .append("</td><td class='txtAlignRight'>")
                    .append(df.format(getTotalDocumentItem(doc)))
                    .append("</td></tr>");
        }
        return ret.toString();
    }

    private float getTotalDocumentItem(DocumentItem doc) {
        float ret = doc.getQuantity() * (doc.getProductPrice() * ((100 + doc.getTaxPercentage())/100) * (100 - doc.getDiscountPercentage())/100);
        billTotal += ret;
        return ret;
    }

    @Override
    protected String getTemplateName() {
        return TEMPLATE_NAME;
    }
}
