package com.uoc.taller.adapter.reports.labels;


import javax.ejb.Singleton;
import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonString;
import javax.json.JsonValue;
import javax.json.JsonValue.ValueType;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.function.BiPredicate;
import java.util.function.Predicate;
import java.util.stream.Collectors;

@Singleton
public class LabelManager {

    private static final String LABELS = "labels.json";

    private final Map<String, Map<String, Object>> reportTypes;
    private BiPredicate<Entry<String, JsonValue>, ValueType> isEntryAValueType = (jsonValue, valueType) -> jsonValue.getValue().getValueType() == valueType;


    public LabelManager() {
        JsonObject labelsFile = Json.createReader(LabelManager.class.getResourceAsStream(LABELS)).readObject();

        Predicate<Entry<String, JsonValue>> elementMustBeJsonTypeObject = (entry) -> isEntryAValueType.test(entry, ValueType.OBJECT);
        reportTypes = labelsFile.entrySet()
                .stream()
                .filter(elementMustBeJsonTypeObject)
                .collect(Collectors.toMap(Entry::getKey, this::createLabelsFromJsonObject));
    }

    private Map<String, Object> createLabelsFromJsonObject(Entry<String, JsonValue> entry) {
        Predicate<Entry<String, JsonValue>> elementMustBeJsonTypeString = (entryStringJsonValue) -> isEntryAValueType.test(entryStringJsonValue, ValueType.STRING);

        return entry.getValue()
                .asJsonObject()
                .entrySet()
                .stream()
                .filter(elementMustBeJsonTypeString)
                .collect(Collectors.toMap(Entry::getKey, subEntry -> ((JsonString) subEntry.getValue()).getString()));
    }


    public String getLabel(String category, String name) {
        return getLabelsByCategory(category).get(name).toString();
    }

    public Map<String, Object> getLabelsByCategory(String category) {
        return reportTypes.get(category);
    }

    public Set<String> getLabelCategories() {
        return reportTypes.keySet();
    }
}
