package com.uoc.taller.adapter.reports.thymeleaf;

import org.thymeleaf.TemplateEngine;
import org.thymeleaf.context.Context;
import org.thymeleaf.templateresolver.ClassLoaderTemplateResolver;

import javax.inject.Inject;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Optional;
import java.util.logging.Logger;

public class HtmlThymeleafTemplateGenerator {


    private static final String TEMPLATE_LOCATION = "/reports/";
    private final TemplateEngine templateEngine;

    @Inject
    private Logger logger;

    public HtmlThymeleafTemplateGenerator() {
        this.templateEngine = new TemplateEngine();
        this.templateEngine.setTemplateResolver(createTemplateResolver());
    }

    private ClassLoaderTemplateResolver createTemplateResolver() {
        ClassLoaderTemplateResolver templateResolver = new ClassLoaderTemplateResolver();
        templateResolver.setCharacterEncoding(StandardCharsets.UTF_8.name());
        templateResolver.setPrefix(TEMPLATE_LOCATION);
        return templateResolver;
    }

    public String processTemplate(String templateName, Context context) {
        return templateEngine.process(templateName, context);
    }

    public Optional<File> processContent(String fileName, String content) {
        File file = new File(fileName + ".html");
        try {
            FileWriter fileWriter = new FileWriter(file);
            fileWriter.write(content);
            fileWriter.flush();
            fileWriter.close();
            return Optional.of(file);
        } catch (IOException e) {
            logger.severe("Error generating HTML. " + e.getMessage());
            return Optional.empty();
        }
    }
}
