package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.ClientDAO;
import com.uoc.taller.adapter.repository.models.ClientDBModel;
import com.uoc.taller.domain.entity.Client;
import com.uoc.taller.domain.port.ClientRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAClientRepository implements ClientRepository {

    private final ClientDAO clientDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAClientRepository(ClientDAO clientDAO, DozerBeanMapper dozerBeanMapper) {
        this.clientDAO = clientDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    public Optional<Client> getClient(Long id) {
        return clientDAO.getClient(id)
                .map(clientDBModel -> dozerBeanMapper.map(clientDBModel, Client.class));
    }

    public List<Client> getAllClients() {
        return clientDAO.getAll()
                .stream()
                .map(element -> dozerBeanMapper.map(element, Client.class))
                .collect(Collectors.toList());
    }


    public Long persist(Client client) {
        return clientDAO.persist(dozerBeanMapper.map(client, ClientDBModel.class));
    }

    @Override
    public Client update(Long id, Client client) {
        ClientDBModel clientDBModel = dozerBeanMapper.map(client, ClientDBModel.class);
        clientDBModel.setId(id);
        return dozerBeanMapper.map(clientDAO.update(clientDBModel), Client.class);
    }
}
