package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.FlowDAO;
import com.uoc.taller.adapter.repository.models.FlowDBModel;
import com.uoc.taller.domain.entity.Flow;
import com.uoc.taller.domain.entity.FlowStatus;
import com.uoc.taller.domain.port.FlowRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAFlowRepository implements FlowRepository {
    private final FlowDAO flowDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAFlowRepository(FlowDAO flowDAO, DozerBeanMapper dozerBeanMapper) {
        this.flowDAO = flowDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    @Override
    public Optional<Flow> getFlow(Long id) {
        return flowDAO.getFlow(id)
                .map(flowDBModel -> dozerBeanMapper.map(flowDBModel, Flow.class));
    }

    @Override
    public List<Flow> getByTallerIdAndStatus(Long tallerId, FlowStatus flowStatus) {
        return flowDAO.getByTallerIdAndStatus(tallerId, flowStatus)
                .stream()
                .map(element -> dozerBeanMapper.map(element, Flow.class))
                .collect(Collectors.toList());
    }

    @Override
    public List<Flow> getByStatus(FlowStatus flowStatus) {
        return flowStatus == null ? getAll() : getFlowByStatus(flowStatus);
    }

    private List<Flow> getFlowByStatus(FlowStatus flowStatus) {
        return flowDAO.getByStatus(flowStatus)
                .stream()
                .map(element -> dozerBeanMapper.map(element, Flow.class))
                .collect(Collectors.toList());
    }

    @Override
    public Long persist(Flow flow) {
        return flowDAO.persist(dozerBeanMapper.map(flow, FlowDBModel.class));
    }

    @Override
    public Flow update(Long id, Flow flow) {
        FlowDBModel flowDBModel = dozerBeanMapper.map(flow, FlowDBModel.class);
        flowDBModel.setId(id);
        return dozerBeanMapper.map(flowDAO.update(flowDBModel), Flow.class);
    }

    @Override
    public List<Flow> getByPlate(String plate) {
        return flowDAO.getByPlate(plate)
                .stream()
                .map(element -> dozerBeanMapper.map(element, Flow.class))
                .collect(Collectors.toList());
    }

    public List<Flow> getAll() {
        return flowDAO.getAll()
                .stream()
                .map(element -> dozerBeanMapper.map(element, Flow.class))
                .collect(Collectors.toList());
    }
}
