package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.GuardTicketDAO;
import com.uoc.taller.adapter.repository.models.GuardTicketDBModel;
import com.uoc.taller.domain.entity.GuardTicket;
import com.uoc.taller.domain.port.GuardTicketRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAGuardTicketRepository implements GuardTicketRepository {

    private final GuardTicketDAO guardTicketDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAGuardTicketRepository(GuardTicketDAO guardTicketDAO, DozerBeanMapper dozerBeanMapper) {
        this.guardTicketDAO = guardTicketDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }


    @Override
    public List<GuardTicket> getAllGuardTicketsByPlate(String plate) {
        return guardTicketDAO.getGuardTicketsByPlate(plate).stream()
                .map(element -> dozerBeanMapper.map(element, GuardTicket.class))
                .collect(Collectors.toList());
    }

    @Override
    public Optional<GuardTicket> getGuardTicket(Long id) {
        return guardTicketDAO.getGuardTicket(id)
                .map(invoiceDBModel -> dozerBeanMapper.map(invoiceDBModel, GuardTicket.class));
    }

    @Override
    public Long persist(GuardTicket guardTicket) {
        return guardTicketDAO.persist(dozerBeanMapper.map(guardTicket, GuardTicketDBModel.class));
    }
}
