package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.InvoiceDAO;
import com.uoc.taller.adapter.repository.models.InvoiceDBModel;
import com.uoc.taller.domain.entity.Invoice;
import com.uoc.taller.domain.port.InvoiceRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAInvoiceRepository implements InvoiceRepository {

    private final InvoiceDAO invoiceDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAInvoiceRepository(InvoiceDAO invoiceDAO, DozerBeanMapper dozerBeanMapper) {
        this.invoiceDAO = invoiceDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    @Override
    public List<Invoice> getAllInvoicesByPlate(String plate) {
        return invoiceDAO.getInvoicesByPlate(plate).stream()
                .map(element -> dozerBeanMapper.map(element, Invoice.class))
                .collect(Collectors.toList());
    }

    @Override
    public Optional<Invoice> getInvoice(Long id) {
            return invoiceDAO.getInvoice(id)
                    .map(invoiceDBModel -> dozerBeanMapper.map(invoiceDBModel, Invoice.class));
    }

    @Override
    public Long persist(Invoice guardTicket) {
        return invoiceDAO.persist(dozerBeanMapper.map(guardTicket, InvoiceDBModel.class));
    }
}
