package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.ProductDAO;
import com.uoc.taller.adapter.repository.models.ProductDBModel;
import com.uoc.taller.domain.entity.Product;
import com.uoc.taller.domain.port.ProductRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAProductRepository implements ProductRepository {

    private final ProductDAO productDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAProductRepository(ProductDAO productDAO, DozerBeanMapper dozerBeanMapper) {
        this.productDAO = productDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    @Override
    public Optional<Product> getProduct(Long id) {
        return productDAO.getProduct(id)
                .map(productDBModel -> dozerBeanMapper.map(productDBModel, Product.class));
    }

    @Override
    public List<Product> getAllProducts() {
        return productDAO.getAll()
                .stream()
                .map(element -> dozerBeanMapper.map(element, Product.class))
                .collect(Collectors.toList());
    }

    @Override
    public Long persist(Product product) {
        return productDAO.persist(dozerBeanMapper.map(product, ProductDBModel.class));
    }

    @Override
    public Product update(Long id, Product product) {
        ProductDBModel productDBModel = dozerBeanMapper.map(product, ProductDBModel.class);
        productDBModel.setId(id);
        return dozerBeanMapper.map(productDAO.update(productDBModel), Product.class);
    }

    @Override
    public List<Product> getByDesc(String desc) {
        return productDAO.getByDesc(desc)
                .stream()
                .map(element -> dozerBeanMapper.map(element, Product.class))
                .collect(Collectors.toList());
    }


}
