package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.TallerDAO;
import com.uoc.taller.adapter.repository.models.TallerDBModel;
import com.uoc.taller.domain.entity.Taller;
import com.uoc.taller.domain.entity.Vehicle;
import com.uoc.taller.domain.port.TallerRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPATallerRepository implements TallerRepository {
    private final TallerDAO tallerDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPATallerRepository(TallerDAO tallerDAO, DozerBeanMapper dozerBeanMapper) {
        this.tallerDAO = tallerDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    @Override
    public Optional<Taller> getTaller(Long id) {
        return tallerDAO.getTaller(id).map(tallerDBModel -> dozerBeanMapper.map(tallerDBModel, Taller.class));
    }

    @Override
    public Long persist(Taller taller) {
        return tallerDAO.persist(dozerBeanMapper.map(taller, TallerDBModel.class));
    }

    @Override
    public Vehicle update(Long id, Taller taller) {
        TallerDBModel tallerDBModel = dozerBeanMapper.map(taller, TallerDBModel.class);
        tallerDBModel.setId(id);
        return dozerBeanMapper.map(tallerDAO.update(tallerDBModel), Vehicle.class);
    }

    @Override
    public List<Taller> getTalleres() {
        return tallerDAO.getAll()
                .stream()
                .map(element -> dozerBeanMapper.map(element, Taller.class))
                .collect(Collectors.toList());
    }
}
