package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.VehicleDAO;
import com.uoc.taller.adapter.repository.models.VehicleDBModel;
import com.uoc.taller.domain.entity.Vehicle;
import com.uoc.taller.domain.port.VehicleRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAVehicleRepository implements VehicleRepository {

    private final VehicleDAO vehicleDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAVehicleRepository(VehicleDAO vehicleDAO, DozerBeanMapper dozerBeanMapper) {
        this.vehicleDAO = vehicleDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    @Override
    public Optional<Vehicle> getVehicle(String plate) {
        return vehicleDAO.getVehicle(plate)
                .map(vehicleDBModel -> dozerBeanMapper.map(vehicleDBModel, Vehicle.class));
    }

    @Override
    public List<Vehicle> getAllVehicles() {
        return vehicleDAO.getAll()
                .stream()
                .map(element -> dozerBeanMapper.map(element, Vehicle.class))
                .collect(Collectors.toList());
    }


    @Override
    public String persist(Vehicle client) {
        return vehicleDAO.persist(dozerBeanMapper.map(client, VehicleDBModel.class));
    }

    @Override
    public List<Vehicle> getByClientId(Long id) {
        return vehicleDAO.getByClientId(id)
                .stream()
                .map(element -> dozerBeanMapper.map(element, Vehicle.class))
                .collect(Collectors.toList());
    }

    @Override
    public Vehicle update(String plate, Vehicle vehicle) {
        VehicleDBModel vehicleDBModel = dozerBeanMapper.map(vehicle, VehicleDBModel.class);
        vehicleDBModel.setPlate(plate);
        return dozerBeanMapper.map(vehicleDAO.update(vehicleDBModel), Vehicle.class);
    }
}
