package com.uoc.taller.adapter.repository;

import com.uoc.taller.adapter.repository.daos.WorkOrderDAO;
import com.uoc.taller.adapter.repository.models.WorkOrderDBModel;
import com.uoc.taller.domain.entity.WorkOrder;
import com.uoc.taller.domain.port.WorkOrderRepository;
import org.dozer.DozerBeanMapper;

import javax.inject.Inject;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

public class JPAWorkOrderRepository implements WorkOrderRepository {

    private final WorkOrderDAO workOrderDAO;
    private final DozerBeanMapper dozerBeanMapper;

    @Inject
    public JPAWorkOrderRepository(WorkOrderDAO workOrderDAO, DozerBeanMapper dozerBeanMapper) {
        this.workOrderDAO = workOrderDAO;
        this.dozerBeanMapper = dozerBeanMapper;
    }

    @Override
    public List<WorkOrder> getAllWorkOrdersByPlate(String plate) {
        return workOrderDAO.getWorkOrdersByPlate(plate).stream()
                .map(element -> dozerBeanMapper.map(element, WorkOrder.class))
                .collect(Collectors.toList());
    }

    @Override
    public Optional<WorkOrder> getWorkOrder(long id) {
        return workOrderDAO.getWorkOrder(id)
                .map(workOrderDBModel -> dozerBeanMapper.map(workOrderDBModel, WorkOrder.class));
    }

    @Override
    public Long persist(WorkOrder workOrder) {
        return workOrderDAO.persist(dozerBeanMapper.map(workOrder, WorkOrderDBModel.class));
    }
}
