package com.uoc.taller.adapter.repository.daos;

import com.uoc.taller.adapter.repository.models.FlowDBModel;
import com.uoc.taller.domain.entity.FlowStatus;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.List;
import java.util.Optional;

public class FlowDAO {
    @PersistenceContext(unitName = "taller")
    private EntityManager entityManager;

    public void setEntityManager(EntityManager entityManager) {
        this.entityManager = entityManager;
    }

    public Optional<FlowDBModel> getFlow(Long id) {
        return Optional.ofNullable(entityManager.find(FlowDBModel.class, id));
    }

    public Long persist(FlowDBModel flowDBModel) {
        entityManager.persist(flowDBModel);
        return flowDBModel.getId();
    }

    public FlowDBModel update(FlowDBModel flowDBModel) {
        return entityManager.merge(flowDBModel);
    }

    public List<FlowDBModel> getAll() {
        return entityManager.createNamedQuery("Flow.findAll", FlowDBModel.class)
                .getResultList();
    }

    public List<FlowDBModel> getByStatus(FlowStatus flowStatus) {
        return entityManager.createNamedQuery("Flow.findByStatus", FlowDBModel.class)
                .setParameter("status", flowStatus)
                .getResultList();
    }

    public List<FlowDBModel> getByTallerIdAndStatus(Long tallerId, FlowStatus flowStatus) {
        return entityManager.createNamedQuery("Flow.findByTallerIdAndStatus", FlowDBModel.class)
                .setParameter("tallerId", tallerId)
                .setParameter("status", flowStatus).getResultList();
    }

    public List<FlowDBModel> getByPlate(String plate) {
        return entityManager.createNamedQuery("Flow.findByPlate", FlowDBModel.class)
                .setParameter("plate", plate)
                .getResultList();
    }
}
