package com.uoc.taller.adapter.repository.models;

import com.uoc.taller.domain.entity.FlowStatus;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToOne;
import java.util.Objects;
import java.util.StringJoiner;

@Entity
@NamedQueries({
        @NamedQuery(name = "Flow.findByStatus", query = "SELECT c FROM FlowDBModel c WHERE c.status = :status"),
        @NamedQuery(name = "Flow.findByTallerIdAndStatus", query = "SELECT f FROM FlowDBModel f JOIN f.guardTicketDBModel g JOIN g.taller t ON t.id = :tallerId WHERE f.status = :status"),
        @NamedQuery(name = "Flow.findAll", query = "SELECT c FROM FlowDBModel c"),
        @NamedQuery(name = "Flow.findByPlate", query = "SELECT c FROM FlowDBModel c WHERE c.guardTicketDBModel.vehicle.plate = :plate ORDER by c.id desc")
})
public class FlowDBModel {

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Long id;

    @OneToOne(fetch = FetchType.EAGER)
    private GuardTicketDBModel guardTicketDBModel;

    @OneToOne(fetch = FetchType.EAGER)
    private WorkOrderDBModel workOrderDBModel;

    @OneToOne(fetch = FetchType.EAGER)
    private InvoiceDBModel invoiceDBModel;

    @Enumerated(EnumType.STRING)
    private FlowStatus status;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public GuardTicketDBModel getGuardTicketDBModel() {
        return guardTicketDBModel;
    }

    public void setGuardTicketDBModel(GuardTicketDBModel guardTicketDBModel) {
        this.guardTicketDBModel = guardTicketDBModel;
    }

    public WorkOrderDBModel getWorkOrderDBModel() {
        return workOrderDBModel;
    }

    public void setWorkOrderDBModel(WorkOrderDBModel workOrderDBModel) {
        this.workOrderDBModel = workOrderDBModel;
    }

    public InvoiceDBModel getInvoiceDBModel() {
        return invoiceDBModel;
    }

    public void setInvoiceDBModel(InvoiceDBModel invoiceDBModel) {
        this.invoiceDBModel = invoiceDBModel;
    }

    public FlowStatus getStatus() {
        return status;
    }

    public void setStatus(FlowStatus status) {
        this.status = status;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        FlowDBModel that = (FlowDBModel) o;
        return Objects.equals(getId(), that.getId());
    }

    @Override
    public int hashCode() {
        return Objects.hash(getId());
    }

    @Override
    public String toString() {
        return new StringJoiner(", ", FlowDBModel.class.getSimpleName() + "[", "]")
                .add("id=" + id)
                .add("guardTicketDBModel=" + guardTicketDBModel)
                .add("workOrderDBModel=" + workOrderDBModel)
                .add("invoiceDBModel=" + invoiceDBModel)
                .add("status=" + status)
                .toString();
    }
}
