package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.CustomMediaType;
import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Client;
import com.uoc.taller.usecase.CreateClient;
import com.uoc.taller.usecase.GetClient;
import com.uoc.taller.usecase.GetVehicle;
import com.uoc.taller.usecase.UpdateClient;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

@Stateless
@Path("/clients")
@Produces(CustomMediaType.JSON_WITH_UTF8_CHARSET)
public class ClientResource {

    private static final String ID = "id";

    @Inject
    private GetClient getClient;

    @Inject
    private CreateClient createClient;

    @Inject
    private UpdateClient updateClient;

    @Inject
    private GetVehicle getVehicle;


    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return getClient.get(id)
                .map(client -> Response.ok(client)
//                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Status.NOT_FOUND)
                        .build());
    }

    @GET
    @SecurityChecked
    public Response getAll() {
        return Response
                .ok(getClient.getAll())
//                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(Client client) {
        return Response
                .status(Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createClient.create(client))
                        .build())
                .build();
    }

    @PUT
    @Path("{id}")
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response update(@PathParam("id") Long id, Client client) {
        return Response
                .status(Status.OK)
                .entity(updateClient.update(id, client))
                .build();
    }

    @GET
    @Path("{id}/vehicles")
    public Response getByClientId(@PathParam("id") Long id) {
        return Response
                .ok(getVehicle.getByClientId(id))
                .build();
    }
}
