package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Flow;
import com.uoc.taller.domain.entity.FlowStatus;
import com.uoc.taller.usecase.CreateFlow;
import com.uoc.taller.usecase.GetFlow;
import com.uoc.taller.usecase.UpdateFlow;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/flows")
@Produces(MediaType.APPLICATION_JSON)
public class FlowResource {

    private static final String ID = "id";
    private static final String STATUS = "status";

    @Inject
    private GetFlow getFlow;

    @Inject
    private CreateFlow createFlow;

    @Inject
    private UpdateFlow updateFlow;

    @GET
    @SecurityChecked
    public Response getAll(@QueryParam(STATUS) FlowStatus flowStatus, @QueryParam("tallerId") Long tallerId) {
        return Response
                .ok(getFlow.getByStatus(flowStatus, tallerId))
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return Response
                .ok(getFlow.get(id))
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(Flow flow) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createFlow.create(flow))
                        .build())
                .build();
    }

    @PUT
    @Path("{id}")
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response update(@PathParam(ID) Long id, Flow flow) {
        return Response
                .status(Response.Status.OK)
                .entity(updateFlow.update(id, flow))
                .build();
    }
}
