package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.GuardTicket;
import com.uoc.taller.usecase.CreateGuardTicket;
import com.uoc.taller.usecase.GenerateGuardTicketHtml;
import com.uoc.taller.usecase.GenerateGuardTicketPdf;
import com.uoc.taller.usecase.GetGuardTicket;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/guard-tickets")
@Produces(MediaType.APPLICATION_JSON)
public class GuardTicketResource {
    private static final String ID = "id";
    private static final String VEHICLE_ID = "plate";

    @Inject
    private GetGuardTicket getGuardTicket;

    @Inject
    private CreateGuardTicket createGuardTicket;

    @Inject
    private GenerateGuardTicketPdf generateGuardTicketPdf;

    @Inject
    private GenerateGuardTicketHtml generateGuardTicketHtml;

    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return getGuardTicket.get(id)
                .map(guardTicket -> Response.ok(guardTicket).header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @SecurityChecked
    public Response getAllByPlate(@QueryParam(VEHICLE_ID) String plate) {
        return Response
                .ok(getGuardTicket.getAllByPlate(plate))
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }


    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(GuardTicket guardTicket) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createGuardTicket.create(guardTicket))
                        .build())
                .build();
    }

    @GET
    @Path("{id}/pdf")
    @SecurityChecked
    public Response getWorkOrderPdf(@PathParam(ID) Long id) {
        return generateGuardTicketPdf.generateGuardTicket(id)
                .map(file -> Response.
                        ok(file)
                        .header("Content-type","application/pdf;charset=UTF-8")
                        .header("Content-Disposition", "inline")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @Path("{id}/html")
    @SecurityChecked
    public Response getWorkOrderHtml(@PathParam(ID) Long id) {
        return generateGuardTicketHtml.generateGuardTicket(id)
                .map(file -> Response.
                        ok(file)
                        .header("Content-type","application/xml;charset=UTF-8")
                        .header("Content-Disposition", "inline")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }
}
