package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Invoice;
import com.uoc.taller.usecase.CreateInvoice;
import com.uoc.taller.usecase.GenerateInvoiceHtml;
import com.uoc.taller.usecase.GenerateInvoicePdf;
import com.uoc.taller.usecase.GetInvoice;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/invoices")
@Produces(MediaType.APPLICATION_JSON)
public class InvoiceResource {

    private static final String ID = "id";
    private static final String VEHICLE_ID = "plate";

    @Inject
    private GetInvoice getInvoice;

    @Inject
    private CreateInvoice createInvoice;

    @Inject
    private GenerateInvoiceHtml generateInvoiceHtml;

    @Inject
    private GenerateInvoicePdf generateInvoicePdf;

    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return getInvoice.get(id)
                .map(invoice -> Response.ok(invoice)
                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @SecurityChecked
    public Response getAllByPlate(@QueryParam(VEHICLE_ID) String plate) {
        return Response
                .ok(getInvoice.getAll(plate))
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(Invoice invoice) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createInvoice.create(invoice))
                        .build())
                .build();
    }

    @GET
    @Path("{id}/pdf")
    @SecurityChecked
    public Response getWorkOrderPdf(@PathParam(ID) Long id) {
        return generateInvoicePdf.generateInvoice(id)
                .map(file -> Response.
                        ok(file)
                        .header("Content-type","application/pdf;charset=UTF-8")
                        .header("Content-Disposition", "inline")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @Path("{id}/html")
    @SecurityChecked
    public Response getWorkOrderHtml(@PathParam(ID) Long id) {
        return generateInvoiceHtml.generateInvoice(id)
                .map(file -> Response.
                        ok(file)
                        .header("Content-type","application/xml;charset=UTF-8")
                        .header("Content-Disposition", "inline")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }
}
