package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Communication;
import com.uoc.taller.domain.entity.FlowStatus;
import com.uoc.taller.usecase.SendAuthorNotification;
import com.uoc.taller.usecase.SendCustomerNotification;
import com.uoc.taller.usecase.SendFlowNotification;

import javax.inject.Inject;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.Response;

@Path("notifications")
public class NotificationResource {

    @Inject
    private SendCustomerNotification sendCustomerNotification;

    @Inject
    private SendAuthorNotification sendAuthorNotification;

    @Inject
    private SendFlowNotification sendFlowNotification;

    @POST
    @Path("flow/{id}")
    @SecurityChecked
    public Response sendFlowNotification(@PathParam("id") Long flowId, @QueryParam("status") FlowStatus flowStatus) {
        return sendFlowNotification.send(flowId, flowStatus) ? Response.ok(true).build() : Response.ok(false).build();
    }

    @POST
    @Path("client/{id}")
    @SecurityChecked
    public Response sendCommunication(@PathParam("id") Long clientId, Communication communication) {
        return sendCustomerNotification.send(clientId, communication) ? Response.ok(true).build() : Response.ok(false).build();
    }

    @POST
    @Path("author")
    @SecurityChecked
    public Response sendCommunication(Communication communication) {
        return sendAuthorNotification.send(communication) ? Response.ok(true).build() : Response.ok(false).build();
    }
}
