package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Product;
import com.uoc.taller.usecase.CreateProduct;
import com.uoc.taller.usecase.GetProduct;
import com.uoc.taller.usecase.UpdateProduct;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.*;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/products")
@Produces(MediaType.APPLICATION_JSON)
public class ProductResource {

    private static final String ID = "id";
    private static final String DESC = "desc";

    @Inject
    private GetProduct getProduct;

    @Inject
    private CreateProduct createProduct;

    @Inject
    private UpdateProduct updateProduct;

    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return getProduct.get(id)
                .map(product -> Response.ok(product)
                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @SecurityChecked
    public Response getAll() {
        return Response
                .ok(getProduct.getAll())
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @GET
    @Path("/search")
    @SecurityChecked
    public Response getByDescription(@QueryParam(DESC) String description) {
        return Response
                .ok(getProduct.getByDescription(description))
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @POST
    @SecurityChecked
    @Consumes(MediaType.APPLICATION_JSON)
    public Response persist(Product product) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createProduct.create(product))
                        .build())
                .build();
    }

    @PUT
    @Path("{id}")
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response update(@PathParam("id") Long id, Product product) {
        return Response
                .status(Response.Status.OK)
                .entity(updateProduct.update(id, product))
                .build();
    }
}
