package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Taller;
import com.uoc.taller.usecase.CreateTaller;
import com.uoc.taller.usecase.GetTaller;
import com.uoc.taller.usecase.UpdateTaller;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/talleres")
@Produces(MediaType.APPLICATION_JSON)
public class TallerResource {
    private static final String ID = "id";

    @Inject
    private GetTaller getTaller;

    @Inject
    private CreateTaller createTaller;

    @Inject
    private UpdateTaller updateTaller;

    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return getTaller.get(id)
                .map(taller -> Response.ok(taller)
                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @SecurityChecked
    public Response getAll() {
        return Response
                .ok(getTaller.getAll())
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(Taller taller) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createTaller.create(taller))
                        .build())
                .build();
    }

    @PUT
    @Path("{id}")
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response update(@PathParam("id") Long id, Taller taller) {
        return Response
                .status(Response.Status.OK)
                .entity(updateTaller.update(id, taller))
                .build();
    }
}
