package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.User;
import com.uoc.taller.domain.entity.exceptions.EncryptionException;
import com.uoc.taller.usecase.AuthenticateUser;
import com.uoc.taller.usecase.CreateUser;
import com.uoc.taller.usecase.GetUser;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.Response.Status;

@Stateless
@Path("users")
public class UserResource {

    private static final String USERNAME = "username";

    @Inject
    private GetUser getUser;

    @Inject
    private CreateUser createUser;

    @Inject
    private AuthenticateUser authenticateUser;

    @GET
    @Path("{username}")
    @SecurityChecked
    public Response get(@PathParam(USERNAME) String loginName) {
        return getUser.get(loginName)
                .map(user -> Response.ok(user)
                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @POST
    @Path("login")
    public Response login(User user) {
        return authenticateUser.validate(user)
                .map(userFound -> Response.ok(userFound)
                        .build())
                .orElse(Response.status(Status.UNAUTHORIZED).build());
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(User user) {
        try {
            return Response
                    .status(Status.CREATED)
                    .entity(Json.createObjectBuilder()
                            .add(USERNAME, createUser.create(user))
                            .build())
                    .build();
        } catch (EncryptionException e) {
            return Response
                    .status(Status.INTERNAL_SERVER_ERROR)
                    .entity(Json.createObjectBuilder()
                            .add("message", e.getMessage())
                            .build())
                    .build();
        }
    }
}
