package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.Vehicle;
import com.uoc.taller.usecase.CreateVehicle;
import com.uoc.taller.usecase.GetFlow;
import com.uoc.taller.usecase.GetVehicle;
import com.uoc.taller.usecase.UpdateVehicle;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/vehicles")
@Produces(MediaType.APPLICATION_JSON)
public class VehicleResource {

    private static final String PLATE = "plate";

    @Inject
    private GetVehicle getVehicle;

    @Inject
    private CreateVehicle createVehicle;

    @Inject
    private UpdateVehicle updateVehicle;

    @Inject
    private GetFlow getFlow;

    @GET
    @Path("{plate}")
    public Response get(@PathParam(PLATE) String plate) {
        return getVehicle.get(plate)
                .map(vehicle -> Response.ok(vehicle)
                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @Path("{plate}/flows")
    public Response getFlowsByPlate(@PathParam(PLATE) String plate) {
        return Response
                .ok(getFlow.getByPlate(plate))
                .build();
    }

    @GET
    @SecurityChecked
    public Response getAll() {
        return Response
                .ok(getVehicle.getAll())
                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(Vehicle vehicle) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(PLATE, createVehicle.create(vehicle))
                        .build())
                .build();
    }

    @PUT
    @Path("{plate}")
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response update(@PathParam("plate") String plate, Vehicle vehicle) {
        return Response
                .status(Response.Status.OK)
                .entity(updateVehicle.update(plate, vehicle))
                .build();
    }
}
