package com.uoc.taller.adapter.rest;

import com.uoc.taller.adapter.rest.config.CustomMediaType;
import com.uoc.taller.adapter.rest.config.SecurityChecked;
import com.uoc.taller.domain.entity.WorkOrder;
import com.uoc.taller.usecase.CreateWorkOrder;
import com.uoc.taller.usecase.GenerateWorkOrderHtml;
import com.uoc.taller.usecase.GenerateWorkOrderPdf;
import com.uoc.taller.usecase.GetWorkOrder;

import javax.ejb.Stateless;
import javax.inject.Inject;
import javax.json.Json;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;

@Stateless
@Path("/work-orders")
@Produces(CustomMediaType.JSON_WITH_UTF8_CHARSET)
public class WorkOrderResource {
    private static final String ID = "id";
    private static final String VEHICLE_ID = "plate";

    @Inject
    private GetWorkOrder getWorkOrder;

    @Inject
    private CreateWorkOrder createWorkOrder;

    @Inject
    private GenerateWorkOrderPdf generateWorkOrderPdf;

    @Inject
    private GenerateWorkOrderHtml generateWorkOrderHtml;

    @GET
    @Path("{id}")
    @SecurityChecked
    public Response get(@PathParam(ID) Long id) {
        return getWorkOrder.get(id)
                .map(guardTicket -> Response.ok(guardTicket)
//                        .header("content-type", "application/json;charset=UTF-8")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @SecurityChecked
    public Response getAllByPlate(@QueryParam(VEHICLE_ID) String plate) {
        return Response
                .ok(getWorkOrder.getAll(plate))
//                .header("content-type", "application/json;charset=UTF-8")
                .build();
    }


    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @SecurityChecked
    public Response persist(WorkOrder workOrder) {
        return Response
                .status(Response.Status.CREATED)
                .entity(Json.createObjectBuilder()
                        .add(ID, createWorkOrder.create(workOrder))
                        .build())
                .build();
    }

    @GET
    @Path("{id}/pdf")
    @SecurityChecked
    @Produces(CustomMediaType.PDF_WITH_UTF8_CHARSET)
    public Response getWorkOrderPdf(@PathParam(ID) Long id) {
        return generateWorkOrderPdf.generateWorkOrder(id)
                .map(file -> Response.
                        ok(file)
//                        .header("Content-type","application/pdf;charset=UTF-8")
                        .header("Content-Disposition", "inline")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }

    @GET
    @Path("{id}/html")
    @SecurityChecked
    @Produces(CustomMediaType.XML_WITH_UTF8_CHARSET)
    public Response getWorkOrderHtml(@PathParam(ID) Long id) {
        return generateWorkOrderHtml.generateWorkOrder(id)
                .map(file -> Response.
                        ok(file)
//                        .header("Content-type","application/xml;charset=UTF-8")
                        .header("Content-Disposition", "inline")
                        .build())
                .orElse(Response.status(Response.Status.NOT_FOUND)
                        .build());
    }


}
