package com.uoc.taller.adapter.rest.config;

import com.uoc.taller.usecase.ValidateUserCredentials;

import javax.inject.Inject;
import javax.ws.rs.container.ContainerRequestContext;
import javax.ws.rs.container.ContainerRequestFilter;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import javax.ws.rs.ext.Provider;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.List;

@Provider
@SecurityChecked
public class AuthRequestFilter implements ContainerRequestFilter {
    private final static String AUTHORIZATION = "Authorization";

    @Inject
    private ValidateUserCredentials validateUserCredentials;

    @Override
    public void filter(ContainerRequestContext requestContext){
        MultivaluedMap<String, String> headersMap = requestContext.getHeaders();
        List<String> auth = headersMap.get(AUTHORIZATION);

        if (auth != null && !auth.isEmpty()) {
            byte[] credDecoded = Base64.getDecoder().decode(auth.get(0).split(" ")[1]);
            String credentials = new String(credDecoded, StandardCharsets.UTF_8);
            String[] values = credentials.split(":", 2);
            if (!validateUserCredentials.validate(values[0], values[1])) {
                requestContext.abortWith(Response.status(Response.Status.UNAUTHORIZED).build());
            }
        } else {
            requestContext.abortWith(Response.status(Response.Status.UNAUTHORIZED).build());
        }
    }
}
