package com.uoc.taller.config.initialdata;

import com.uoc.taller.domain.entity.Client;
import com.uoc.taller.domain.entity.Document;
import com.uoc.taller.domain.entity.DocumentItem;
import com.uoc.taller.domain.entity.Flow;
import com.uoc.taller.domain.entity.FlowStatus;
import com.uoc.taller.domain.entity.GuardTicket;
import com.uoc.taller.domain.entity.Invoice;
import com.uoc.taller.domain.entity.PaymentMethod;
import com.uoc.taller.domain.entity.Product;
import com.uoc.taller.domain.entity.ProductCategory;
import com.uoc.taller.domain.entity.Taller;
import com.uoc.taller.domain.entity.User;
import com.uoc.taller.domain.entity.Vehicle;
import com.uoc.taller.domain.entity.WorkOrder;
import com.uoc.taller.domain.entity.exceptions.EncryptionException;
import com.uoc.taller.usecase.CreateClient;
import com.uoc.taller.usecase.CreateFlow;
import com.uoc.taller.usecase.CreateGuardTicket;
import com.uoc.taller.usecase.CreateInvoice;
import com.uoc.taller.usecase.CreateProduct;
import com.uoc.taller.usecase.CreateTaller;
import com.uoc.taller.usecase.CreateUser;
import com.uoc.taller.usecase.CreateVehicle;
import com.uoc.taller.usecase.CreateWorkOrder;
import com.uoc.taller.usecase.GetClient;
import com.uoc.taller.usecase.GetGuardTicket;
import com.uoc.taller.usecase.GetInvoice;
import com.uoc.taller.usecase.GetProduct;
import com.uoc.taller.usecase.GetTaller;
import com.uoc.taller.usecase.GetVehicle;
import com.uoc.taller.usecase.GetWorkOrder;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;
import javax.inject.Inject;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;

@Startup
@Singleton
public class LoadData {
    @Inject
    private CreateClient createClient;
    @Inject
    private CreateFlow createFlow;
    @Inject
    private CreateGuardTicket createGuardTicket;
    @Inject
    private CreateInvoice createInvoice;
    @Inject
    private CreateProduct createProduct;
    @Inject
    private CreateTaller createTaller;
    @Inject
    private CreateUser createUser;
    @Inject
    private CreateVehicle createVehicle;
    @Inject
    private CreateWorkOrder createWorkOrder;
    @Inject
    private GetVehicle getVehicle;
    @Inject
    private GetGuardTicket getGuardTicket;
    @Inject
    private GetWorkOrder getWorkOrder;
    @Inject
    private GetInvoice getInvoice;
    @Inject
    private GetTaller getTaller;
    @Inject
    private GetClient getClient;
    @Inject
    private GetProduct getProduct;
    @Inject
    private Logger logger;


    private Random random;

    public LoadData() {
        random = new Random();
    }

    @PostConstruct
    public void runLoader() {
        Long tallerId = createTaller.create(funcCreateTaller());
        Flow flow = null;
        try {
            createUser.create(funcCreateUser());
        } catch (EncryptionException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
        }
        for (int i = 0; i < 5; i++) {
            Long clientId = createClient.create(funcCreateClient());
            String plate = createVehicle.create(funcCreateVehicle(clientId));
            Long guardTicketId = createGuardTicket.create(funcCreateGuardTicket(plate, tallerId, getClient.get(clientId).get()));
            if (i < 2) {
                flow = funcCreateFlow(
                        getGuardTicket.get(guardTicketId).get(),
                        null,
                        null,
                        FlowStatus.GUARDTICKET);
            } else {
                Long workOrderId = createWorkOrder.create(funcCreateWorkOrder(plate, tallerId, getClient.get(clientId).get()));
                if (i < 4) {
                    flow = funcCreateFlow(
                            getGuardTicket.get(guardTicketId).get(),
                            getWorkOrder.get(workOrderId).get(),
                            null,
                            FlowStatus.WORKORDER);
                } else {
                    Long invoiceId = createInvoice.create(funcCreateInvoice(plate, tallerId, getClient.get(clientId).get()));
                    flow = funcCreateFlow(
                            getGuardTicket.get(guardTicketId).get(),
                            getWorkOrder.get(workOrderId).get(),
                            getInvoice.get(invoiceId).get(),
                            FlowStatus.CLOSED);
                }
            }
            createFlow.create(flow);
        }
    }

    public Flow funcCreateFlow(GuardTicket guardTicket, WorkOrder workOrder, Invoice invoice, FlowStatus status) {
        Flow flow = new Flow();
        flow.setGuardTicket(guardTicket);
        flow.setWorkOrder(workOrder);
        flow.setInvoice(invoice);
        flow.setStatus(status);
        return flow;
    }

    public Client funcCreateClient() {
        Client client = new Client();
        client.setAddress(AddressesEnum.getRandomAddress());
        //-3600000 == 01/01/1970 https://codechi.com/dev-tools/date-to-millisecond-calculators/
        client.setBirthday(new Date(-3600000L + (Math.abs(random.nextLong()) % (20L * 365 * 24 * 60 * 60 * 1000))));
        client.setCity(CitiesEnum.getRandomCity());
        client.setCountry("España");
        client.setEmail(EmailsEnum.getRandomEmail());
        client.setName(NamesEnum.getRandomName());
        client.setSurname(SurnamesEnum.getRandomSurName() + " " + SurnamesEnum.getRandomSurName());
        client.setNif(new Random().nextInt(89999999) + 10000000 + "A");
        client.setPostalCode("330" + (new Random().nextInt(50) + 10));
        client.setProvince("Asturias");
        client.setTaxName(client.getName() + " " + client.getSurname());
        client.setTelephone("985" + (new Random().nextInt(899999) + 100000));
        return client;
    }

    public Vehicle funcCreateVehicle(Long clientId) {
        Vehicle vehicle = new Vehicle();
        vehicle.setClient(getClient.get(clientId).get());
        vehicle.setBrand(BrandsEnum.getRandomBrandName());
        vehicle.setChassisNumber(new Random().nextInt(8999999) + 1000000 + "BC");
        vehicle.setColor(ColorsEnum.getRandomColorName());
        vehicle.setPlate(new Random().nextInt(8999) + 1000 + "FJI");
        vehicle.setYear(String.valueOf(new Random().nextInt(20) + 1989));
        vehicle.setModel(ModelsEnum.getRandomModelName());
        return vehicle;
    }

    public Product funcCreateProduct() {
        Product product = new Product();
        product.setAvailableStock(new Random().nextInt(200));
        ProductCategory productCategory = ProductCategory.getRandomProductCategory();
        product.setProductCategory(productCategory);
        product.setDescription("Descripción del producto" + new Random().nextInt(180));
        product.setDiscountPercentage(new Random().nextInt(20) + 1f);
        product.setPrice(new Random().nextInt(180) + 1f);
        product.setTaxPercentage(new Random().nextInt(20) + 1f);
        product.setCode(product.getProductCategory().toString() + new Random().nextInt(20));
        return product;
    }

    public DocumentItem funcCreateDocumentItem(Long prodId) {
        DocumentItem documentItem = new DocumentItem();
        documentItem.setDiscountPercentage(new Random().nextInt(20) + 1f);
        documentItem.setProductPrice(new Random().nextInt(180) + 1f);
        documentItem.setQuantity(new Random().nextInt(4) + 1);
        documentItem.setTaxPercentage(new Random().nextInt(20) + 1f);
        documentItem.setProduct(getProduct.get(prodId).get());
        return documentItem;
    }

    public Taller funcCreateTaller() {
        Taller taller = new Taller();
        taller.setAddress(AddressesEnum.getRandomAddress());
        taller.setCity(CitiesEnum.getRandomCity());
        taller.setCountry("España");
        taller.setProvince("Asturias");
        taller.setEmail(EmailsEnum.getRandomEmail());
        taller.setName(TallerNamesEnum.getRandomTallerName());
        taller.setNif(new Random().nextInt(89999999) + 10000000 + "A");
        taller.setPostalCode("330" + (new Random().nextInt(50) + 10));
        taller.setTaxName(taller.getName());
        taller.setTelephone("985" + (new Random().nextInt(899999) + 100000));
        return taller;
    }

    public User funcCreateUser() {
        User user = new User();
        user.setLoginName("David");
        user.setName(NamesEnum.getRandomName());
        user.setSurname(SurnamesEnum.getRandomSurName() + " " + SurnamesEnum.getRandomSurName());
        user.setPassword("123");
        return user;
    }

    public Invoice funcCreateInvoice(String plate, Long tallerId, Client client) {
        Invoice invoice = (Invoice) getDocument(new Invoice(), plate, tallerId, client);
        invoice.setPaymentMethod(PaymentMethod.getRandomPaymentMethod());

        invoice.setDocumentItems(generateDocumentItemDBModelList());
        return invoice;
    }

    public List<DocumentItem> generateDocumentItemDBModelList() {
        List<Long> productListIds = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            productListIds.add(createProduct.create(funcCreateProduct()));
        }
        List<DocumentItem> documentItemList = new ArrayList<>();
        for (int i = 0; i < 5; i++) {
            documentItemList.add(funcCreateDocumentItem(productListIds.get(i)));
        }
        return documentItemList;
    }

    public WorkOrder funcCreateWorkOrder(String plate, Long tallerId, Client client) {
        WorkOrder workOrder = (WorkOrder) getDocument(new WorkOrder(), plate, tallerId, client);
        workOrder.setDocumentItems(generateDocumentItemDBModelList());
        LocalDate futureDate = LocalDate.now().plusMonths(1);
        workOrder.setEndValidity(Date.from(futureDate.atStartOfDay().atZone(ZoneId.systemDefault()).toInstant()));
        return workOrder;
    }

    public GuardTicket funcCreateGuardTicket(String plate, Long tallerId, Client client) {
        return (GuardTicket) getDocument(new GuardTicket(), plate, tallerId, client);
    }

    private Document getDocument(Document document, String plate, Long tallerId, Client client) {
        document.setDateOfIssue(new Date());
        document.setExpectedDate(new Date(new Date().getTime() + (24 * 60 * 60 * 1000 * new Random().nextInt(50))));
        document.setUserFiscalName(client.getTaxName());
        document.setNumberKilometers(new Random().nextInt(40000));
        document.setObservations("Observaciones varias de la factura");
        document.setVehicle(getVehicle.get(plate).get());
        document.setTaller(getTaller.get(tallerId).get());
        document.setUserAddress(client.getAddress());
        document.setUserCity(client.getCity());
        document.setUserCountry(client.getCountry());
        document.setUserProvince(client.getProvince());
        document.setUserPostalCode(client.getPostalCode());
        document.setUserTelephone(client.getTelephone());
        document.setUserMail(client.getEmail());
        return document;
    }
}
