package com.uoc.taller.usecase;

import com.uoc.taller.domain.entity.Communication;
import com.uoc.taller.domain.entity.Flow;
import com.uoc.taller.domain.entity.FlowStatus;
import com.uoc.taller.domain.port.EmailSender;

import javax.inject.Inject;
import java.util.Optional;


public class SendFlowNotification {

    @Inject
    private GetFlow getFlow;

    @Inject
    private EmailSender emailSender;

    @Inject
    private GenerateGuardTicketPdf generateGuardTicketPdf;

    @Inject
    private GenerateInvoicePdf generateInvoicePdf;

    @Inject
    private GenerateWorkOrderPdf generateWorkOrderPdf;

    public boolean send(Long flowId, FlowStatus flowStatus) {
        Optional<Flow> optionalFlow = getFlow.get(flowId);
        if (optionalFlow.isPresent()) {
            Flow flow = optionalFlow.get();
            return processFlowStatus(flow, flowStatus)
                    .map(communication -> emailSender.sendEmail(getEmail(flow), communication.getSubject(), communication.getBody(), communication.getAttachment()))
                    .orElse(false);
        }
        return false;
    }

    private String getEmail(Flow flow) {
        if (flow.getGuardTicket() != null) {
            return flow.getGuardTicket().getVehicle().getClient().getEmail();
        }
        return "";
    }

    private Optional<Communication> processFlowStatus(Flow flow, FlowStatus flowStatus) {

        switch (flowStatus) {
            case CLOSED:
                return Optional.of(generateInvoicePdf.generateInvoice(flow.getInvoice().getId())
                        .map(file -> {
                            Communication communication = new Communication();
                            communication.setSubject("Su factura está disponible");
                            communication.setBody("Hola " + flow.getInvoice().getUserFiscalName() + " su factura está disponible");
                            communication.setAttachments(file);
                            return communication;
                        })
                        .orElseGet(() -> {
                            Communication communication = new Communication();
                            communication.setSubject("Su factura está disponible");
                            communication.setBody("Hola " + flow.getInvoice().getUserFiscalName() + " su factura NO está disponible");
                            return communication;
                        }));
            case WORKORDER:
                return Optional.of(generateWorkOrderPdf.generateWorkOrder(flow.getWorkOrder().getId())
                        .map(file -> {
                            Communication communication = new Communication();
                            communication.setSubject("Su orden de reparación está disponible");
                            communication.setBody("Hola " + flow.getWorkOrder().getUserFiscalName() + " su orden de reparación está disponible");
                            communication.setAttachments(file);
                            return communication;
                        })
                        .orElseGet(() -> {
                            Communication communication = new Communication();
                            communication.setSubject("Su orden de reparación está disponible");
                            communication.setBody("Hola " + flow.getWorkOrder().getUserFiscalName() + " su orden de reparación NO está disponible");
                            return communication;
                        }));
            case GUARDTICKET:
                return Optional.of(generateGuardTicketPdf.generateGuardTicket(flow.getGuardTicket().getId())
                        .map(file -> {
                            Communication communication = new Communication();
                            communication.setSubject("Su resguardo está disponible");
                            communication.setBody("Hola " + flow.getGuardTicket().getUserFiscalName() + " su resguardo está disponible");
                            communication.setAttachments(file);
                            return communication;
                        })
                        .orElseGet(() -> {
                            Communication communication = new Communication();
                            communication.setSubject("Su resguardo está disponible");
                            communication.setBody("Hola " + flow.getGuardTicket().getUserFiscalName() + " su resguardo NO está disponible");
                            return communication;
                        }));
            default:
                return Optional.empty();
        }
    }
}
