package com.uoc.taller.usecase;

import com.uoc.taller.domain.entity.exceptions.EncryptionException;
import com.uoc.taller.domain.port.EncryptionService;
import com.uoc.taller.domain.port.UserRepository;

import javax.inject.Inject;

public class ValidateUserCredentials {

    private final UserRepository userRepository;
    private final EncryptionService encryptionService;

    @Inject
    public ValidateUserCredentials(UserRepository userRepository, EncryptionService encryptionService) {
        this.userRepository = userRepository;
        this.encryptionService = encryptionService;
    }

    public boolean validate(String userName, String password) {
        return userRepository.getUser(userName)
                .map(user -> checkPassword(user.getPassword(), password))
                .orElse(false);
    }

    private boolean checkPassword(String storedPassword, String password) {
        try {
            return encryptionService.validate(storedPassword, password);
        } catch (EncryptionException e) {
            return false;
        }
    }
}
