import { NgModule } from '@angular/core';
import { Routes, RouterModule } from '@angular/router';
import { GuardTicketDashboardComponent }   from './guard-ticket/guard-ticket-dashboard/guard-ticket-dashboard.component';
import { WorkOrderDashboardComponent }   from './work-order/work-order-dashboard/work-order-dashboard.component';
import { InvoiceDashboardComponent } from './invoice/invoice-dashboard/invoice-dashboard.component';
import { ClientsComponent } from './clients/clients.component';
import { ClientDetailComponent } from './clients/client-detail/client-detail.component';
import { ClientFormComponent } from './clients/client-form/client-form.component';
import { VehiclesComponent } from './vehicles/vehicles.component';
import { VehicleFormComponent } from './vehicles/vehicle-form/vehicle-form.component';
import { VehicleDetailComponent } from './vehicles/vehicle-detail/vehicle-detail.component';
import { ProductsComponent } from './products/products.component';
import { ProductDetailComponent } from './products/product-detail/product-detail.component';
import { ProductFormComponent } from './products/product-form/product-form.component';
import { TalleresComponent } from './talleres/talleres.component';
import { TallerDetailComponent } from './talleres/taller-detail/taller-detail.component';
import { TallerFormComponent } from './talleres/taller-form/taller-form.component';
import { CreateGuardTicketComponent } from './guard-ticket/create-guard-ticket/create-guard-ticket.component';
import { CreateWorkOrderComponent } from './work-order/create-work-order/create-work-order.component';
import { CreateInvoiceComponent } from './invoice/create-invoice/create-invoice.component';
import { LoginComponent } from './login/login.component';
import { AuthGuard } from './helpers/auth.guard';
import { GuardTicketPreviewComponent } from './guard-ticket/guard-ticket-preview/guard-ticket-preview.component';
import { WorkOrderPreviewComponent } from './work-order/work-order-preview/work-order-preview.component';
import { InvoicePreviewComponent } from './invoice/invoice-preview/invoice-preview.component';



const routes: Routes = [
  { path: '', redirectTo: '/guard-tickets', pathMatch: 'full' },
  { path: 'guard-tickets', component: GuardTicketDashboardComponent, canActivate: [AuthGuard]}, 
  { path: 'add-guard-ticket', component: CreateGuardTicketComponent, canActivate: [AuthGuard]}, 
  { path: 'guard-tickets/:id', component: GuardTicketPreviewComponent , canActivate: [AuthGuard]},
  { path: 'invoices', component: InvoiceDashboardComponent, canActivate: [AuthGuard]},
  { path: 'invoices/:id', component: InvoicePreviewComponent , canActivate: [AuthGuard]},
  { path: 'add-invoice', component: CreateInvoiceComponent, canActivate: [AuthGuard]}, 
  { path: 'work-orders', component: WorkOrderDashboardComponent , canActivate: [AuthGuard]},
  { path: 'add-work-order', component: CreateWorkOrderComponent, canActivate: [AuthGuard]},
  { path: 'work-orders/:id', component: WorkOrderPreviewComponent , canActivate: [AuthGuard]},
  { path: 'clients', component: ClientsComponent, canActivate: [AuthGuard]},
  { path: 'clients/:id', component: ClientDetailComponent , canActivate: [AuthGuard]},
  { path: 'add-client', component: ClientFormComponent , canActivate: [AuthGuard]},
  { path: 'vehicles', component: VehiclesComponent , canActivate: [AuthGuard]},
  { path: 'add-vehicle', component: VehicleFormComponent , canActivate: [AuthGuard]},
  { path: 'vehicles/:plate', component: VehicleDetailComponent , canActivate: [AuthGuard]},
  { path: 'products', component: ProductsComponent , canActivate: [AuthGuard]},
  { path: 'products/:id', component: ProductDetailComponent , canActivate: [AuthGuard]},
  { path: 'add-product', component: ProductFormComponent , canActivate: [AuthGuard]},
  { path: 'talleres', component: TalleresComponent , canActivate: [AuthGuard]},
  { path: 'talleres/:id', component: TallerDetailComponent , canActivate: [AuthGuard]},
  { path: 'talleres/:id/edit', component: TallerDetailComponent , canActivate: [AuthGuard]},
  { path: 'add-taller', component: TallerFormComponent , canActivate: [AuthGuard]},
  { path: 'login', component: LoginComponent}
];


@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule { }
