import { BrowserModule } from '@angular/platform-browser';
import { NgModule } from '@angular/core';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';

import { AppRoutingModule } from './app-routing.module';
import { AppComponent } from './app.component';
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { GuardTicketDashboardComponent } from './guard-ticket/guard-ticket-dashboard/guard-ticket-dashboard.component';
import { WorkOrderDashboardComponent } from './work-order/work-order-dashboard/work-order-dashboard.component';
import { InvoiceDashboardComponent } from './invoice/invoice-dashboard/invoice-dashboard.component';
import { ClientsComponent } from './clients/clients.component';
import { ClientDetailComponent } from './clients/client-detail/client-detail.component';
import { VehiclesComponent } from './vehicles/vehicles.component';
import { VehicleDetailComponent } from './vehicles/vehicle-detail/vehicle-detail.component';
import { ProductsComponent } from './products/products.component';
import { ProductDetailComponent } from './products/product-detail/product-detail.component';
import { ProductFormComponent } from './products/product-form/product-form.component';
import { MenuComponent } from './menu/menu.component';
import { ClientFormComponent } from './clients/client-form/client-form.component';
import { VehicleFormComponent } from './vehicles/vehicle-form/vehicle-form.component';
import { TalleresComponent } from './talleres/talleres.component';
import { TallerDetailComponent } from './talleres/taller-detail/taller-detail.component';
import { CreateGuardTicketComponent } from './guard-ticket/create-guard-ticket/create-guard-ticket.component';
import { FindVehicleComponent } from './finders/find-vehicle/find-vehicle.component';
import { CreateWorkOrderComponent } from './work-order/create-work-order/create-work-order.component';
import { FindProductComponent } from './finders/find-product/find-product.component';
import { HtmlDateComponent } from './html-components/html-date/html-date.component';
import { HtmlDateTimeComponent } from './html-components/html-date-time/html-date-time.component';
import { CreateInvoiceComponent } from './invoice/create-invoice/create-invoice.component';
import { ClientSheetComponent } from './clients/client-sheet/client-sheet.component';
import { LoginComponent } from './login/login.component';
import { ErrorInterceptor } from './helpers/error.interceptor';
import { LogoutComponent } from './header/logout/logout.component';
import { SessionComponent } from './header/session/session.component';
import { HeaderComponent } from './header/header.component';
import { TallerComponent } from './header/taller/taller.component';
import { TallerFormComponent } from './talleres/taller-form/taller-form.component';
import { EmailValidator } from './directives/email-validator';
import { NifValidator } from './directives/nif-validator';
import { SelectValidator } from './directives/select-validator';
import { NgxPaginationModule } from 'ngx-pagination';
import { ConfirmationDialogComponent } from './html-components/confirmation-dialog/confirmation-dialog.component';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';
import { GuardTicketPreviewComponent } from './guard-ticket/guard-ticket-preview/guard-ticket-preview.component';
import { WorkOrderPreviewComponent } from './work-order/work-order-preview/work-order-preview.component';
import { InvoicePreviewComponent } from './invoice/invoice-preview/invoice-preview.component';
import { SendEmailComponent } from './html-components/send-email/send-email.component';
import { FooterComponent } from './footer/footer.component';

@NgModule({
  declarations: [
    AppComponent,
    GuardTicketDashboardComponent,
    WorkOrderDashboardComponent,
    InvoiceDashboardComponent,
    ClientsComponent,
    ClientDetailComponent,
    VehiclesComponent,
    VehicleDetailComponent,
    ProductsComponent,
    ProductDetailComponent,
    ProductFormComponent,
    MenuComponent,
    ClientFormComponent,
    VehicleFormComponent,
    TalleresComponent,
    TallerDetailComponent,
    CreateGuardTicketComponent,
    FindVehicleComponent,
    CreateWorkOrderComponent,
    FindProductComponent,
    HtmlDateComponent,
    HtmlDateTimeComponent,
    CreateInvoiceComponent,
    ClientSheetComponent,
    LoginComponent,
    LogoutComponent,
    SessionComponent,
    HeaderComponent,
    TallerComponent,
    TallerFormComponent,
    EmailValidator,
    NifValidator,
    SelectValidator,
    ConfirmationDialogComponent,
    GuardTicketPreviewComponent,
    WorkOrderPreviewComponent,
    InvoicePreviewComponent,
    SendEmailComponent,
    FooterComponent
  ],
  imports: [
    BrowserModule,
    AppRoutingModule,
    HttpClientModule,
    FormsModule,
    ReactiveFormsModule,
    NgxPaginationModule,
    NgbModule.forRoot()
  ],
  providers: [
    { provide: HTTP_INTERCEPTORS, useClass: ErrorInterceptor, multi: true }
  ],
  bootstrap: [AppComponent],
  entryComponents: [ ConfirmationDialogComponent, SendEmailComponent]
})
export class AppModule { }
