import {Component, OnInit} from '@angular/core';
import {Client} from '../services/clients/client';
import {ClientService} from '../services/clients/client.service';
import {OrderService} from '../services/util/order.service';
import {SendEmailService} from '../services/send-email.service';
import {NotificationService} from '../services/notifications/notification.service';

@Component({
  selector: 'app-clients',
  templateUrl: './clients.component.html',
  styleUrls: ['./clients.component.scss']
})
export class ClientsComponent implements OnInit {

  clientList: Client[];
  filteredList: Client[];
  queryToFilter: string;
  shownPage: number = 1;
  order: string;

  constructor(
    private clientService: ClientService,
    private orderService: OrderService,
    private sendEmailService: SendEmailService,
    private notificationService: NotificationService) {
  }

  ngOnInit() {
    this.getClients();
    this.order = "desc";
  }

  getClients(): void {
    this.clientService.getClients()
      .subscribe(clients => {
        this.clientList = clients;
        this.assignCopy();
      });
  }

  assignCopy() {
    this.filteredList = Object.assign([], this.clientList);
  }

  filterItem() {
    if (!this.queryToFilter) {
      this.assignCopy();
    }

    this.filteredList = Object.assign([], this.clientList)
      .filter(item => (item.name + " " + item.surname).toLowerCase().indexOf(this.queryToFilter.toLowerCase()) > -1)
  }

  sort(property: string) {
    let tempOrder: string;
    this.order == "desc" ? tempOrder = "asc" : tempOrder = "desc";
    this.filteredList.sort(this.orderService.compareValues(property, tempOrder));
    this.order = tempOrder;
  }


  public sendEmail(idClient: number, emailTo: string) {
    this.sendEmailService.send(emailTo)
      .then((confirmed) => {
        if (confirmed && confirmed.length == 2) {
          this.notificationService.sendEmailToClient(idClient, confirmed[0], confirmed[1])
            .subscribe(data => console.log("Email enviado"), error => console.log("Error email no enviado"));
        }
      })
      .catch(() => console.log('User dismissed the dialog (e.g., by using ESC, clicking the cross icon, or clicking outside the dialog)'));
  }


}
