import { Component, OnInit, Output, EventEmitter, ViewChild, ElementRef } from '@angular/core';
import { Product } from 'src/app/services/products/product';
import { ProductService } from 'src/app/services/products/product.service';
import { Observable, Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';


@Component({
  selector: 'app-find-product',
  templateUrl: './find-product.component.html',
  styleUrls: ['./find-product.component.scss']
})
export class FindProductComponent implements OnInit {

  desc: string;

  @ViewChild('searchInput') searchInput: ElementRef;

  @Output()
  productFound = new EventEmitter<Product>();

  results: Product[];
  searchTerm$ = new Subject<string>();

  constructor(private productService: ProductService) {
    this.search(this.searchTerm$)
      .subscribe(results => {
        this.results = results;
      });
  }

  ngOnInit() {
  }

  search(terms: Observable<string>) {
    return terms.pipe(debounceTime(400),
      distinctUntilChanged(),
      switchMap(term => this.findProductsByDescription(term))
    );
  }

  findProductsByDescription(description: any): Observable<Product[]> {
    return this.productService.getProductByDescription(description);
  }

  addProduct(product: Product) {
    this.productFound.emit(product);
    this.results = [];
    this.searchInput.nativeElement.value = "";
  }
}
