import { Component, OnInit } from '@angular/core';
import { GuardTicket } from 'src/app/services/guard-tickets/guard-ticket';
import { Location } from '@angular/common';
import { Taller } from 'src/app/services/talleres/taller';
import { TallerService } from 'src/app/services/talleres/taller.service';
import { Vehicle } from 'src/app/services/vehicles/vehicle';
import { GuardTicketService } from 'src/app/services/guard-tickets/guard-ticket.service';
import { FlowService } from 'src/app/services/flows/flow.service';
import { Flow } from 'src/app/services/flows/flow';
import { VehicleFinderResults } from 'src/app/finders/find-vehicle/vehicle-finder-result';
import { ActivatedRoute } from '@angular/router';
import { map } from 'rxjs/operators';
import { PROVINCES } from '../../enums/provinces';

const GTStatus = "GUARDTICKET";


@Component({
  selector: 'app-create-guard-ticket',
  templateUrl: './create-guard-ticket.component.html',
  styleUrls: ['./create-guard-ticket.component.scss']
})
export class CreateGuardTicketComponent implements OnInit {

  model = new GuardTicket();
  vehicle: Vehicle;
  tallerList: Taller[];
  labelDateOfIssue: string;
  idDateOfIssue: string;
  labelExpectedDate: string;
  idExpectedDate: string;
  vehicleFound: boolean;
  plate: string;
  nowDate: string;
  provinces = PROVINCES;

  constructor(
    private location: Location,
    private tallerService: TallerService,
    private guardTicketService: GuardTicketService,
    private flowService: FlowService,
    public activatedRoute: ActivatedRoute
  ) { }

  ngOnInit() {
    this.activatedRoute.paramMap
      .pipe(map(() => window.history.state))
      .subscribe(data => {
        this.model.vehicle = data.vehicle;
      });
    this.nowDate = new Date().getTime().toString();
    this.tallerService.getTalleres().subscribe(talleres => this.tallerList = talleres);
    this.labelDateOfIssue = "Fecha de expedición";
    this.idDateOfIssue = "dateOfIssue";
    this.labelExpectedDate = "Fecha prevista de entrega";
    this.idExpectedDate = "expectedDate";
    this.vehicleFound = true;
    this.model.taller = new Taller();
  }

  onSubmit() {
    this.guardTicketService
      .createGuardTicket(this.model)
      .subscribe(
        guardTicket => {
          this.model = guardTicket;
          this.createFlow();
        });
    //this.createFlow();
  }

  createFlow() {
    let flow = new Flow();
    flow.status = GTStatus;
    flow.guardTicket = new GuardTicket();
    flow.guardTicket.id = this.model.id;
    this.flowService
      .createFlow(flow)
      .subscribe(
        flow => {
          this.goBack();
        });
  }

  goBack(): void {
    this.location.back();
  }

  saveData(vehicleFinder: VehicleFinderResults) {
    if (vehicleFinder.vehicle != null) {
      this.vehicle = vehicleFinder.vehicle;
      let vehiclePost = new Vehicle();
      vehiclePost.plate = vehicleFinder.vehicle.plate;
      this.model.vehicle = vehiclePost;
      this.vehicleFound = true;
    } else {
      this.vehicleFound = false;
      this.model.vehicle = null;
      this.plate = vehicleFinder.plate;
    }
  }

  saveDate(date: number, idField: string) {
    switch (idField) {
      case this.idDateOfIssue:
        this.model.dateOfIssue = date;
        break;
      case this.idExpectedDate:
        this.model.expectedDate = date;
        break;
    }
  }

}
