import { Component, OnInit } from '@angular/core';
import { FlowService } from '../../services/flows/flow.service';
import { Flow } from '../../services/flows/flow';
import { GuardTicketService } from 'src/app/services/guard-tickets/guard-ticket.service';
import { PdfReportService } from 'src/app/services/pdf-report/pdf-report.service';
import { ConfirmationDialogService } from 'src/app/services/confirmation-dialog.service';
import { NotificationService } from 'src/app/services/notifications/notification.service';

const CANCELED_FLOW = "CANCELED";
const MOBILE_SIZE = 768;

@Component({
  selector: 'app-guard-ticket-dashboard',
  templateUrl: './guard-ticket-dashboard.component.html',
  styleUrls: ['./guard-ticket-dashboard.component.scss']
})
export class GuardTicketDashboardComponent implements OnInit {


  guardTicketFlows: Flow[];
  shownPage: number = 1;
  filteredList: Flow[];
  queryToFilter: string;
  isCollapsed: boolean[] = [];
  pagination: number = 10;

  constructor(
    private flowService: FlowService,
    private guardTicketService: GuardTicketService,
    private pdfReportService: PdfReportService,
    private confirmationDialogService: ConfirmationDialogService,
    private notificationService: NotificationService) {

  }

  loadGuardTicketFlows() {
    this.flowService.getGuardTicketFlows()
      .subscribe(flows => {
        this.guardTicketFlows = flows;
        this.assignCopy();
      });
  }

  ngOnInit() {
    this.loadGuardTicketFlows();
    this.loadCollapseStatus();
  }

  getPDF(id: number) {
    this.pdfReportService.getPDF(id, this.guardTicketService);
  }

  getHTML(id: number) {
    return this.guardTicketService.getHTML(id)
      .subscribe(data => console.log(data)),
      error => console.log('Error viewing the file.'),
      () => console.info('OK');
  }

  public openConfirmationDialog(position: number) {
    this.confirmationDialogService.confirm('¿Estas seguro?', '¿De verdad deseas cancelar el flujo?')
      .then((confirmed) => {
        if (confirmed) {
          this.cancel(position);
        }
      })
      .catch(() => console.log('User dismissed the dialog (e.g., by using ESC, clicking the cross icon, or clicking outside the dialog)'));
  }

  cancel(position: number) {
    this.guardTicketFlows[position].status = CANCELED_FLOW;
    this.flowService.updateFlow(this.guardTicketFlows[position]).subscribe(
      flow => {
        this.loadGuardTicketFlows();
      });
  }

  private loadCollapseStatus() {
    this.treatCollapse(window.innerWidth < MOBILE_SIZE ? (): boolean => true : (): boolean => false);
  }

  onCollapse(index: number) {
    this.treatCollapse((i: number): boolean => index == i ? !this.isCollapsed[i] : true);
  }

  private treatCollapse(collapseBehavior: (index: number) => boolean) {
    for (let i: number = 0; i < this.pagination; i++) {
      this.isCollapsed[i] = collapseBehavior(i);
    }
  }

  assignCopy() {
    this.filteredList = Object.assign([], this.guardTicketFlows);
  }

  filterItem() {
    if (!this.queryToFilter) {
      this.assignCopy();
    }

    this.filteredList = Object.assign([], this.guardTicketFlows)
      .filter(item => (item.guardTicket.vehicle.plate + "" + item.guardTicket.vehicle.brand + "" + item.guardTicket.vehicle.model).toLowerCase().indexOf(this.queryToFilter.toLowerCase()) > -1)
  }

  sendEmail(id: number) {
    this.notificationService.sendFlow(id, "GUARDTICKET")
      .subscribe(data => data ? console.log("Email sent") : console.log("Error sending email"));
  }

}
