import { Component, OnInit } from '@angular/core';
import { InvoiceService } from 'src/app/services/invoices/invoice.service';
import { FlowService } from 'src/app/services/flows/flow.service';
import { ActivatedRoute } from '@angular/router';
import { map } from 'rxjs/operators';
import { GuardTicket } from 'src/app/services/guard-tickets/guard-ticket';
import { WorkOrder } from 'src/app/services/work-orders/work-order';
import { DocumentItem } from 'src/app/services/document-items/document-item';
import { Invoice } from 'src/app/services/invoices/invoice';
import { Flow } from 'src/app/services/flows/flow';
import { Location } from '@angular/common';
import { Product } from 'src/app/services/products/product';
import { PROVINCES } from '../../enums/provinces';

const INVOICE_STATUS = "CLOSED";
const MOBILE_SIZE = 768;

class ValueAndText {
  constructor(public Value: string, public Text: string) { }
}

@Component({
  selector: 'app-create-invoice',
  templateUrl: './create-invoice.component.html',
  styleUrls: ['./create-invoice.component.scss']
})
export class CreateInvoiceComponent implements OnInit {
  idFlow: number;
  guardTicket: GuardTicket;
  workOrder: WorkOrder;
  documentItemList: DocumentItem[] = [];
  labelDateOfIssue: string;
  idDateOfIssue: string;
  model = new Invoice();
  nowDate: string;
  provinces = PROVINCES;
  isCollapsed: boolean[] = [];

  paymentMethod: ValueAndText[] = [
    new ValueAndText("CARD", "Tarjeta"),
    new ValueAndText("CASH", "Efectivo"),
    new ValueAndText("PAYPAL", "PayPal"),
    new ValueAndText("VERSE", "Verse"),
    new ValueAndText("BIZUM", "Bizum"),
    new ValueAndText("TRANSFER", "Transferencia")
  ];

  constructor(
    private location: Location,
    // private tallerService: TallerService,
    private invoiceService: InvoiceService,
    private flowService: FlowService,
    public activatedRoute: ActivatedRoute) { }

  ngOnInit() {
    this.activatedRoute.paramMap
      .pipe(map(() => window.history.state))
      .subscribe(data => {
        this.idFlow = data.idFlow;
        this.guardTicket = data.guardTicket;
        this.workOrder = data.workOrder;
        this.model.taller = this.workOrder.taller;
        this.model.vehicle = this.workOrder.vehicle;
        this.model.userAddress = this.workOrder.userAddress;
        this.model.userCity = this.workOrder.userCity;
        this.model.userCountry = this.workOrder.userCountry;
        this.model.userFiscalName = this.workOrder.userFiscalName;
        this.model.userMail = this.workOrder.userMail;
        this.model.userPostalCode = this.workOrder.userPostalCode;
        this.model.userProvince = this.workOrder.userProvince;
        this.model.userTelephone = this.workOrder.userTelephone;
        this.model.numberKilometers = this.workOrder.numberKilometers;
        this.documentItemList = this.workOrder.documentItems;
        this.loadCollapseStatus();
      });

    this.nowDate = new Date().getTime().toString();
    this.labelDateOfIssue = "Fecha de expedición";
    this.idDateOfIssue = "dateOfIssue";
    //Detached error if not ids deleted
    this.documentItemList.forEach(function (value) {
      value.id = null;
    });
  }

  saveDate(date: number, idField: string) {
    if (idField == this.idDateOfIssue) {
      this.model.dateOfIssue = date;
    }
    else {
      console.log("Error salvando la fecha de la factura: " + idField)
    }
  }

  saveData(product: Product) {
    let documentItem = new DocumentItem();
    documentItem.productPrice = product.price;
    documentItem.discountPercentage = product.discountPercentage;
    documentItem.taxPercentage = product.taxPercentage;
    documentItem.product = product;
    this.documentItemList.push(documentItem);
  }

  onSubmit() {
    this.model.documentItems = this.documentItemList;
    this.invoiceService
      .createInvoice(this.model)
      .subscribe(
        invoice => {
          this.model = invoice;
          this.updateFlow();
        });
  }

  updateFlow() {
    let flow = new Flow();
    flow.status = INVOICE_STATUS;
    flow.invoice = new Invoice();
    flow.invoice.id = this.model.id;
    flow.workOrder = new WorkOrder();
    flow.workOrder.id = this.workOrder.id;
    flow.guardTicket = new GuardTicket();
    flow.guardTicket.id = this.guardTicket.id;
    flow.id = this.idFlow;
    this.flowService
      .updateFlow(flow)
      .subscribe(
        flow => {
          this.goBack();
        });
  }

  goBack(): void {
    this.location.back();
  }

  calculateTotal(document: DocumentItem) {
    if (document.quantity == null)
      return 0;
    return (document.quantity * (document.productPrice * ((100 + document.taxPercentage) / 100) * (100 - document.discountPercentage) / 100)).toFixed(2);
  }

  calculateTableTotal(): string {
    let ret = 0;
    this.documentItemList.forEach((value) => {
      if (value.quantity != undefined) {
        ret += Number(this.calculateTotal(value));
      }
    });
    return ret.toFixed(2);
  }

  deleteItem(index: number) {
    this.documentItemList.splice(index, 1);
  }

  private loadCollapseStatus() {
    this.treatCollapse(window.innerWidth < MOBILE_SIZE ? (): boolean => true : (): boolean => false);
  }

  onCollapse(index: number) {
    this.treatCollapse((i: number): boolean => index == i ? !this.isCollapsed[i] : true);
  }

  private treatCollapse(collapseBehavior: (index: number) => boolean) {
    for (let i: number = 0; i < this.documentItemList.length; i++) {
      this.isCollapsed[i] = collapseBehavior(i);
    }
  }

}
