import { Component, OnInit } from '@angular/core';
import { FlowService } from '../../services/flows/flow.service';
import { Flow } from '../../services/flows/flow';
import { InvoiceService } from 'src/app/services/invoices/invoice.service';
import { PdfReportService } from 'src/app/services/pdf-report/pdf-report.service';
import { Invoice } from 'src/app/services/invoices/invoice';
import { ConfirmationDialogService } from 'src/app/services/confirmation-dialog.service';
import { NotificationService } from 'src/app/services/notifications/notification.service';

const CANCELED_FLOW = "CANCELED";
const MOBILE_SIZE = 768;

@Component({
  selector: 'app-invoice-dashboard',
  templateUrl: './invoice-dashboard.component.html',
  styleUrls: ['./invoice-dashboard.component.scss']
})
export class InvoiceDashboardComponent implements OnInit {

  invoiceFlows: Flow[];
  shownPage: number = 1;
  filteredList: Flow[];
  queryToFilter: string;
  isCollapsed: boolean[] = [];
  pagination: number = 10;

  constructor(
    private flowService: FlowService,
    private invoiceService: InvoiceService,
    private pdfReportService: PdfReportService,
    private confirmationDialogService: ConfirmationDialogService,
    private notificationService: NotificationService) { }

  loadInvoicesFlows() {
    this.flowService.getFinishedFlows()
      .subscribe(flows => {
        this.invoiceFlows = flows;
        this.assignCopy();
      });
  }

  ngOnInit() {
    this.loadInvoicesFlows();
    this.loadCollapseStatus();
  }

  getTotalPrice(invoice: Invoice): number {
    return this.invoiceService.getTotalPrice(invoice);
  }

  getPDF(id: number) {
    this.pdfReportService.getPDF(id, this.invoiceService);
  }

  getHTML(id: number) {
    return this.invoiceService.getHTML(id)
      .subscribe(data => console.log(data)),
      error => console.log('Error viewing the file.'),
      () => console.info('OK');
  }

  public openConfirmationDialog(position: number) {
    this.confirmationDialogService.confirm('¿Estas seguro?', '¿De verdad deseas cancelar el flujo?')
      .then((confirmed) => {
        if (confirmed) {
          this.cancel(position);
        }
      })
      .catch(() => console.log('User dismissed the dialog (e.g., by using ESC, clicking the cross icon, or clicking outside the dialog)'));
  }

  cancel(position: number) {
    this.invoiceFlows[position].status = CANCELED_FLOW;
    this.flowService.updateFlow(this.invoiceFlows[position]).subscribe(
      flow => {
        this.loadInvoicesFlows();
      });
  }

  private loadCollapseStatus() {
    this.treatCollapse(window.innerWidth < MOBILE_SIZE ? (): boolean => true : (): boolean => false);
  }

  onCollapse(index: number) {
    this.treatCollapse((i: number): boolean => index == i ? !this.isCollapsed[i] : true);
  }

  private treatCollapse(collapseBehavior: (index: number) => boolean) {
    for (let i: number = 0; i < this.pagination; i++) {
      this.isCollapsed[i] = collapseBehavior(i);
    }
  }

  assignCopy() {
    this.filteredList = Object.assign([], this.invoiceFlows);
  }

  filterItem() {
    if (!this.queryToFilter) {
      this.assignCopy();
    }

    this.filteredList = Object.assign([], this.invoiceFlows)
      .filter(item => (item.invoice.vehicle.plate + "" + item.invoice.vehicle.brand + "" + item.invoice.vehicle.model).toLowerCase().indexOf(this.queryToFilter.toLowerCase()) > -1)
  }

  sendEmail(id: number) {
    this.notificationService.sendFlow(id, "CLOSED")
      .subscribe(data => data ? console.log("Email sent") : console.log("Error sending email"));
  }
}
