import { Component, OnInit } from '@angular/core';
import { Product } from '../services/products/product';
import { ProductService } from '../services/products/product.service';
import { OrderService } from '../services/util/order.service';

@Component({
  selector: 'app-products',
  templateUrl: './products.component.html',
  styleUrls: ['./products.component.scss']
})
export class ProductsComponent implements OnInit {
  productList: Product[];
  filteredList: Product[];
  queryToFilterCategory: string;
  queryToFilterDescription: string;
  shownPage: number = 1;
  order: string;

  constructor(
    private productService: ProductService,
    private orderService: OrderService) { }

  ngOnInit() {
    this.getProducts();
    this.order = "desc";
  }

  getProducts(): void {
    this.productService.getProducts()
      .subscribe(products => {
        this.productList = products;
        this.assignCopy();
      });
  }

  assignCopy() {
    this.filteredList = Object.assign([], this.productList);
  }

  filterItem() {
    if (!this.queryToFilterCategory && !this.queryToFilterDescription) {
      this.assignCopy();
    } else {
      this.filteredList = Object.assign([], this.productList)
        .filter(item => item.code.toLowerCase().indexOf(this.queryToFilterCategory == undefined ? '' : this.queryToFilterCategory.toLowerCase()) == 0)
        .filter(item => item.description.toLowerCase().indexOf(this.queryToFilterDescription == undefined ? '' : this.queryToFilterDescription.toLowerCase()) > -1);
    }
  }

  sort(property: string) {
    let tempOrder: string;
    this.order == "desc" ? tempOrder = "asc" : tempOrder = "desc";
    this.filteredList.sort(this.orderService.compareValues(property, tempOrder));
    this.order = tempOrder;
  }

}
