import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { Client } from './client';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { MessageService } from '../message.service';
import { catchError, tap } from 'rxjs/operators';
import { GenericRestService } from '../generic-rest/generic-rest.service';

@Injectable({
  providedIn: 'root'
})
export class ClientService { 

  private clientsURL = this.genericRestService.getUrlPrefix()+ '/clients';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService : GenericRestService
  ) { }

  getClient(id: number): Observable<Client> {
    const url = `${this.clientsURL}/${id}`;
    return this.http.get<Client>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched client id=${id}`)),
      catchError(this.handleError<Client>(`getClient id=${id}`))
    );
  }

  /** GET clients from the server */
  getClients(): Observable<Client[]> {
    return this.http.get<Client[]>(this.clientsURL, this.genericRestService.getHeaders())
      .pipe(
        tap(_ => this.log('fetched clients')),
        catchError(this.handleError<Client[]>('getClients', []))
      );
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  addClient(client: Client): Observable<Client> {
    return this.http.post<Client>(this.clientsURL, client, this.genericRestService.getHeaders()).pipe(
      tap((newClient: Client) => this.log(`added client w/ id=${newClient.id}`)),
      catchError(this.handleError<Client>('addClient')));
  }

  updateClient(client: Client): Observable<Client> {
    return this.http.put<Client>(`${this.clientsURL}/${client.id}`, client, this.genericRestService.getHeaders()).pipe(
      tap((newClient: Client) => this.log(`updated product w/ id=${newClient.id}`)),
      catchError(this.handleError<Client>('addProduct')));
  }

  private log(message: string) {
    this.messageService.add('ClientService: ${message}');
  }

}
