import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { DocumentCommonService } from '../document-common.service';
import { GuardTicket } from './guard-ticket';
import { GenericRestService } from '../generic-rest/generic-rest.service';
import { HttpClient } from '@angular/common/http';
import { catchError, tap } from 'rxjs/operators';
import { MessageService } from '../message.service';

@Injectable({
  providedIn: 'root'
})
export class GuardTicketService {

  private guardTicketsUrl = this.genericRestService.getUrlPrefix() + '/guard-tickets';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private documentCommonService: DocumentCommonService,
    private genericRestService: GenericRestService) { }

  getPDF(idParam: any): Observable<any> {
    return this.documentCommonService.getPDF(idParam, this.guardTicketsUrl);
  }

  getHTML(idParam: any): Observable<any> {
    return this.documentCommonService.getHTML(idParam, this.guardTicketsUrl);
  }

  getGuardTicket(id: number): Observable<GuardTicket> {
    const url = `${this.guardTicketsUrl}/${id}`;
    return this.http.get<GuardTicket>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched guardticket with id=${id}`)),
      catchError(this.handleError<GuardTicket>(`getGuardTicket id=${id}`))
    );
  }

  createGuardTicket(guardTicket: GuardTicket): Observable<any> {
    return this.http.post<GuardTicket>(this.guardTicketsUrl, guardTicket, this.genericRestService.getHeaders()).pipe(
      tap((newGuardTicket: GuardTicket) => this.log(`added guardTicket w/ id=${newGuardTicket.id}`)),
      catchError(this.handleError<GuardTicket>('createGuardTicket')));
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`ProductService: ${message}`);
  }
}
