import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { Invoice } from './invoice';
import { DocumentCommonService } from '../document-common.service';
import { MessageService } from '../message.service';
import { GenericRestService } from '../generic-rest/generic-rest.service';
import { HttpClient } from '@angular/common/http';
import { catchError, tap } from 'rxjs/operators';

@Injectable({
  providedIn: 'root'
})
export class InvoiceService {

  private invoicesUrl = this.genericRestService.getUrlPrefix() + '/invoices';

  constructor(
    private http: HttpClient,
    private documentCommonService: DocumentCommonService,
    private messageService: MessageService,
    private genericRestService: GenericRestService) { }

  getPDF(idParam: any): Observable<any> {
    return this.documentCommonService.getPDF(idParam, this.invoicesUrl);
  }

  getHTML(idParam: any): Observable<any> {
    return this.documentCommonService.getHTML(idParam, this.invoicesUrl);
  }

  getTotalPrice(object: Invoice): number {
    if (object.documentItems.length > 0) {
      object.totalPrice = 0;
      object.documentItems.forEach(function (value) {
        object.totalPrice += (value.quantity *
          (value.productPrice *
            ((100 + value.taxPercentage) / 100) *
            (100 - value.discountPercentage) / 100
          )
        );
      });
    }
    return object.totalPrice;
  }

  getInvoice(id: number): Observable<Invoice> {
    const url = `${this.invoicesUrl}/${id}`;
    return this.http.get<Invoice>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched invoice with id=${id}`)),
      catchError(this.handleError<Invoice>(`getInvoice id=${id}`))
    );
  }

  createInvoice(invoice: Invoice): Observable<Invoice> {
    return this.http.post<Invoice>(this.invoicesUrl, invoice, this.genericRestService.getHeaders()).pipe(
      tap((newInvoice: Invoice) => this.log(`added invoice w/ id=${newInvoice.id}`)),
      catchError(this.handleError<Invoice>('createInvoice')));
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`ProductService: ${message}`);
  }
}
