import {Injectable} from '@angular/core';
import {HttpClient, HttpResponse} from '@angular/common/http';
import {Observable, of} from 'rxjs';
import {MessageService} from '../message.service';
import {catchError, tap} from 'rxjs/operators';
import {GenericRestService} from '../generic-rest/generic-rest.service';
import {Notification} from './notification';


@Injectable({
  providedIn: 'root'
})
export class NotificationService {

  private notificationsUrl = this.genericRestService.getUrlPrefix() + '/notifications';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService: GenericRestService) {
  }

  sendFlow(idFlow: number, status: string): Observable<boolean> {
    return this.http.post<boolean>(`${this.notificationsUrl}/flow/${idFlow}?status=${status}`, null, this.genericRestService.getHeaders()).pipe(
      tap((response: boolean) => this.log(`sent notification w/ result=${response}`)),
      catchError(this.handleError<boolean>('sendFlow')));
  }

  private log(message: string) {
    this.messageService.add('ClientService: ${message}');
  }

  sendEmailToClient(idClient: number, subject: string, body: string) {
    let notification = new Notification();
    notification.subject = subject;
    notification.body = body;

    return this.http.post<HttpResponse<boolean>>(`${this.notificationsUrl}/client/${idClient}`, notification, this.genericRestService.getHeaders());
  }

  sendEmailToAuthor(subject: string, body: string) {
    let notification = new Notification();
    notification.subject = subject;
    notification.body = body;


    return this.http.post<HttpResponse<boolean>>(`${this.notificationsUrl}/author`, notification, this.genericRestService.getHeaders());
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }


}
