import { Injectable } from '@angular/core';
import {HttpClient} from "@angular/common/http";
import {MessageService} from "../message.service";
import {GenericRestService} from "../generic-rest/generic-rest.service";
import {Observable, of} from "rxjs";
import {Product} from "../products/product";
import {catchError, tap} from "rxjs/operators";

@Injectable({
  providedIn: 'root'
})
export class PlateService {
  private platesUrl = this.genericRestService.getUrlPrefix() + '/plates';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService: GenericRestService
  ) { }

  getPlate(fileToUpload: File): Observable<any> {
    const url = `${this.platesUrl}/recognition`;
    return this.http.post(url, fileToUpload, this.genericRestService.getImageHeaders()).pipe(
      tap(_ => this.log(`fetched image`)),
      catchError(this.handleError<Product>(`fetched image`))
    );
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`ProductService: ${message}`);
  }
}
