import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { Taller } from './taller';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { MessageService } from '../message.service';
import { catchError, tap } from 'rxjs/operators';
import { GenericRestService } from '../generic-rest/generic-rest.service';

@Injectable({
  providedIn: 'root'
})
export class TallerService {


  private talleresURL = this.genericRestService.getUrlPrefix() + '/talleres';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService : GenericRestService
  ) { }

  getTalleres(): Observable<Taller[]> {
    return this.http.get<Taller[]>(this.talleresURL, this.genericRestService.getHeaders())
      .pipe(
        tap(_ => this.log('fetched talleres')),
        catchError(this.handleError<Taller[]>('getTalleres', []))
      );
  }

  getTaller(id: number) {
    const url = `${this.talleresURL}/${id}`;
    return this.http.get<Taller>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched taller id=${id}`)),
      catchError(this.handleError<Taller>(`getTaller id=${id}`))
    );
  }

  addTaller(taller: Taller): Observable<Taller> {
    return this.http.post<Taller>(this.talleresURL, taller, this.genericRestService.getHeaders()).pipe(
      tap((newTaller: Taller) => this.log(`added taller w/ id=${newTaller.id}`)),
      catchError(this.handleError<Taller>('addTaller')));
  }

  updateTaller(taller: Taller): Observable<Taller> {
    return this.http.put<Taller>(`${this.talleresURL}/${taller.id}`, taller, this.genericRestService.getHeaders()).pipe(
      tap((newProduct: Taller) => this.log(`updated taller w/ id=${newProduct.id}`)),
      catchError(this.handleError<Taller>('updateTaller')));
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`ProductService: ${message}`);
  }
}
