﻿import { Injectable } from '@angular/core';
import { HttpClient } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { GenericRestService } from '../generic-rest/generic-rest.service';
import { User } from './users';
import { UserStorage } from './userStorage';
import { Subject } from 'rxjs/internal/Subject';

@Injectable({ providedIn: 'root' })
export class AuthenticationService {
    public isLogged = new Subject();
    
    constructor(
        private http: HttpClient,
        private genericRestService: GenericRestService
    ) { }

    login(username: string, password: string) {
        let user = new User();
        user.loginName = username;
        user.password = password;
        return this.http.post<User>(`${this.genericRestService.getUrlPrefix()}/users/login`, user, this.genericRestService.getHeaders())
            .pipe(map(user => {
                // login successful if there's a user in the response
                let userStorage;
                if (user) {
                    userStorage = this.mapUserStorage(user);
                    // store user details and basic auth credentials in local storage 
                    // to keep user logged in between page refreshes
                    userStorage.authData = window.btoa(username + ':' + password);
                    localStorage.setItem('currentUser', JSON.stringify(userStorage));
                    this.isLogged.next(true);
                }
                return userStorage;
            }));
    }

    mapUserStorage(user: User): UserStorage {
        let userStorage = new UserStorage();
        userStorage.loginName = user.loginName;
        userStorage.name = user.name;
        userStorage.surname = user.surname;
        return userStorage;
    }

    logout() {
        // remove user from local storage to log user out
        localStorage.removeItem('currentUser');
    }



}