import { Injectable } from '@angular/core';
import { Observable,of } from 'rxjs';
import { WorkOrder } from './work-order';
import { DocumentCommonService } from '../document-common.service';
import { GenericRestService } from '../generic-rest/generic-rest.service';
import { MessageService } from '../message.service';
import { HttpClient } from '@angular/common/http';
import { catchError, tap } from 'rxjs/operators';

@Injectable({
  providedIn: 'root'
})
export class WorkOrderService {

  private workOrdersUrl = this.genericRestService.getUrlPrefix() + '/work-orders';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private documentCommonService: DocumentCommonService,
    private genericRestService: GenericRestService) { }

  getPDF(idParam: any): Observable<any> {
    return this.documentCommonService.getPDF(idParam, this.workOrdersUrl);
    // var headers = this.genericRestService.getDocumentHeaders();
    // return this.http.get(this.workOrdersUrl.concat(idParam, '/pdf'), { headers, responseType: 'blob' })
  }

  getHTML(idParam: any): Observable<any> {
    return this.documentCommonService.getHTML(idParam, this.workOrdersUrl);
    // var headers = this.genericRestService.getDocumentHeaders();
    // return this.http.get(this.workOrdersUrl.concat(idParam, '/html'), { headers, responseType: 'text' })
  }

  createWorkOrder(workOrder: WorkOrder): Observable<WorkOrder> {
    return this.http.post<WorkOrder>(this.workOrdersUrl, workOrder, this.genericRestService.getHeaders()).pipe(
      tap((newWorkOrder: WorkOrder) => this.log(`added workOrder w/ id=${newWorkOrder.id}`)),
      catchError(this.handleError<WorkOrder>('createWorkOrderService')));
  }

  getWorkOrder(id: number): Observable<WorkOrder> {
    const url = `${this.workOrdersUrl}/${id}`;
    return this.http.get<WorkOrder>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched workOrder with id=${id}`)),
      catchError(this.handleError<WorkOrder>(`getWorkOrder id=${id}`))
    );
  }

  getTotalPrice(object: WorkOrder): number {
    if (object.documentItems.length > 0) {
      object.totalPrice = 0;
      object.documentItems.forEach(function (value) {
        /*doc.getQuantity * 
        (doc.getProductPrice * ((100 + doc.getTaxPercentage) / 100) * 
        (100 - doc.getDiscountPercentage) / 100)*/
        object.totalPrice += (value.quantity *
          (value.productPrice *
            ((100 + value.taxPercentage) / 100) *
            (100 - value.discountPercentage) / 100
          )
        );
      });
    }
    return object.totalPrice;
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`ProductService: ${message}`);
  }
}
