import { Component, OnInit } from '@angular/core';
import { VehicleService } from '../../services/vehicles/vehicle.service';
import { ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { Vehicle } from '../../services/vehicles/vehicle';
import { Client } from '../../services/clients/client';
import { ClientService } from '../../services/clients/client.service';
import { Flow } from 'src/app/services/flows/flow';
import { FlowService } from 'src/app/services/flows/flow.service';
import { GuardTicketService } from 'src/app/services/guard-tickets/guard-ticket.service';
import { InvoiceService } from 'src/app/services/invoices/invoice.service';
import { WorkOrderService } from 'src/app/services/work-orders/work-order.service';
import { PdfReportService } from 'src/app/services/pdf-report/pdf-report.service';

@Component({
  selector: 'app-vehicle-detail',
  templateUrl: './vehicle-detail.component.html',
  styleUrls: ['./vehicle-detail.component.scss']
})
export class VehicleDetailComponent implements OnInit {
  vehicle: Vehicle;
  clientList: Client[];
  flowList: Flow[];

  constructor(
    private route: ActivatedRoute,
    private vehicleService: VehicleService,
    private clientService: ClientService,
    private flowService: FlowService,
    private workOrderService: WorkOrderService,
    private guardTicketService: GuardTicketService,
    private invoiceService: InvoiceService,
    private pdfReportService: PdfReportService,
    private location: Location) {
    this.clientService.getClients().subscribe(clients => this.clientList = clients);
  }

  ngOnInit(): void {
    this.getVehicle();
  }

  updateClient() {
    if (this.vehicle.client.id != -1) {
      this.clientService.getClient(this.vehicle.client.id).subscribe(client => this.vehicle.client = client);
    }
  }

  getVehicle(): void {
    const plate = this.route.snapshot.paramMap.get('plate');
    this.vehicleService.getVehicle(plate)
      .subscribe(vehicle => {
        this.vehicle = vehicle;
        this.getFlows();
      });
  }

  getFlows() {
    this.flowService.getFlowsByPlate(this.vehicle.plate)
      .subscribe(flowList => this.flowList = flowList);
  }

  onSubmit() {
    this.vehicleService.updateVehicle(this.vehicle).subscribe(vehicle => this.vehicle = vehicle);
    this.goBack();
  }

  goBack(): void {
    this.location.back();
  }

  getPDF(id: number, type: number) {
    switch (type) {
      case 0:
        this.pdfReportService.getPDF(id, this.guardTicketService);
        break;
      case 1:
        this.pdfReportService.getPDF(id, this.workOrderService);
        break;
      case 2:
        this.pdfReportService.getPDF(id, this.invoiceService);
        break;
    }

  }
}
