import { Component, OnInit, Input } from '@angular/core';
import { Vehicle } from '../services/vehicles/vehicle';
import { VehicleService } from '../services/vehicles/vehicle.service';
import { ActivatedRoute } from '@angular/router';
import { map } from 'rxjs/operators';
import { OrderService } from '../services/util/order.service';

@Component({
  selector: 'app-vehicles',
  templateUrl: './vehicles.component.html',
  styleUrls: ['./vehicles.component.scss']
})
export class VehiclesComponent implements OnInit {

  vehicleList: Vehicle[];
  filteredList: Vehicle[];
  queryToFilter: string;
  shownPage: number = 1;
  order: string;

  constructor(
    private vehicleService: VehicleService,
    public activatedRoute: ActivatedRoute,
    private orderService: OrderService
  ) { }

  ngOnInit() {
    this.activatedRoute.paramMap
      .pipe(map(() => window.history.state))
      .subscribe(data => {
        if (data.client == null) {
          this.getVehicles();
        } else {
          this.getVehiclesByClient(data.client.id);
        }
      });
  }

  assignCopy() {
    this.filteredList = Object.assign([], this.vehicleList);
  }

  filterItem() {
    if (!this.queryToFilter) {
      this.assignCopy();
    }
    this.filteredList = Object.assign([], this.vehicleList)
      .filter(item => item.plate.toLowerCase().indexOf(this.queryToFilter.toLowerCase()) == 0)
  }

  getVehiclesByClient(id: number) {
    this.vehicleService.getVehiclesByClientId(id)
      .subscribe(vehicles => {
        this.vehicleList = vehicles;
        this.assignCopy();
      });
  }

  getVehicles() {
    this.vehicleService.getVehicles()
      .subscribe(vehicles => {
        this.vehicleList = vehicles;
        this.assignCopy();
      });
  }

  sort(property: string) {
    let tempOrder: string;
    this.order == "desc" ? tempOrder = "asc" : tempOrder = "desc";
    this.filteredList.sort(this.orderService.compareValues(property, tempOrder));
    this.order = tempOrder;
  }

}
