import { Component, OnInit } from '@angular/core';
import { Location } from '@angular/common';
import { WorkOrder } from 'src/app/services/work-orders/work-order';
import { WorkOrderService } from 'src/app/services/work-orders/work-order.service';
import { FlowService } from 'src/app/services/flows/flow.service';
import { Flow } from 'src/app/services/flows/flow';
import { ActivatedRoute } from '@angular/router';
import { map } from 'rxjs/operators';
import { DocumentItem } from 'src/app/services/document-items/document-item';
import { Product } from 'src/app/services/products/product';
import { GuardTicket } from 'src/app/services/guard-tickets/guard-ticket';
import { PROVINCES } from '../../enums/provinces';

const WO_STATUS = "WORKORDER";
const MOBILE_SIZE = 768;

@Component({
  selector: 'app-create-work-order',
  templateUrl: './create-work-order.component.html',
  styleUrls: ['./create-work-order.component.scss']
})
export class CreateWorkOrderComponent implements OnInit {
  model = new WorkOrder();
  idFlow: number;
  guardTicket: GuardTicket;
  documentItemList: DocumentItem[] = [];
  labelDateOfIssue: string;
  idDateOfIssue: string;
  labelEndValidity: string;
  labelExpectedDate: string;
  idExpectedDate: string;
  nowDate: string;
  provinces = PROVINCES;
  isCollapsed: boolean[] = [];

  constructor(
    private location: Location,
    // private tallerService: TallerService,
    private workOrderService: WorkOrderService,
    private flowService: FlowService,
    public activatedRoute: ActivatedRoute
  ) {
  }

  ngOnInit() {
    //this.tallerService.getTalleres().subscribe(talleres => this.tallerList = talleres);
    this.activatedRoute.paramMap
      .pipe(map(() => window.history.state))
      .subscribe(data => {
        this.idFlow = data.idFlow;
        this.guardTicket = data.guardTicket;
        this.model.taller = this.guardTicket.taller;
        this.model.vehicle = this.guardTicket.vehicle;
        this.model.userAddress = this.guardTicket.userAddress;
        this.model.userCity = this.guardTicket.userCity;
        this.model.userCountry = this.guardTicket.userCountry;
        this.model.userFiscalName = this.guardTicket.userFiscalName;
        this.model.userMail = this.guardTicket.userMail;
        this.model.userPostalCode = this.guardTicket.userPostalCode;
        this.model.userProvince = this.guardTicket.userProvince;
        this.model.userTelephone = this.guardTicket.userTelephone;
        this.model.numberKilometers = this.guardTicket.numberKilometers;
        this.loadCollapseStatus();
      });
    this.nowDate = new Date().getTime().toString();
    this.labelDateOfIssue = "Fecha de expedición";
    this.idDateOfIssue = "dateOfIssue";
    this.labelExpectedDate = "Fecha prevista de entrega";
    this.labelEndValidity = "Fecha de validez";
    this.idExpectedDate = "expectedDate";
  }

  onSubmit() {
    this.model.documentItems = this.documentItemList;
    this.workOrderService
      .createWorkOrder(this.model)
      .subscribe(
        workOrder => {
          this.model = workOrder;
          this.updateFlow();
        });
  }

  updateFlow() {
    let flow = new Flow();
    flow.status = WO_STATUS;
    flow.workOrder = new WorkOrder();
    flow.workOrder.id = this.model.id;
    flow.guardTicket = new GuardTicket();
    flow.guardTicket.id = this.guardTicket.id;
    flow.id = this.idFlow;
    this.flowService
      .updateFlow(flow)
      .subscribe(
        flow => {
          this.goBack();
        });
  }

  goBack(): void {
    this.location.back();
  }

  saveData(product: Product) {
    let documentItem = new DocumentItem();
    documentItem.productPrice = product.price;
    documentItem.discountPercentage = product.discountPercentage;
    documentItem.taxPercentage = product.taxPercentage;
    documentItem.product = product;
    this.documentItemList.push(documentItem);
  }

  saveDate(date: number, idField: string) {
    switch (idField) {
      case this.idDateOfIssue:
        this.model.dateOfIssue = date;
        break;
      case this.idExpectedDate:
        this.model.expectedDate = date;
        break;
    }
  }

  /*doc.getQuantity * 
         (doc.getProductPrice * ((100 + doc.getTaxPercentage) / 100) * 
         (100 - doc.getDiscountPercentage) / 100)*/
  calculateTotal(document: DocumentItem) {
    if (document.quantity == null)
      return 0;
    return (document.quantity * (document.productPrice * ((100 + document.taxPercentage) / 100) * (100 - document.discountPercentage) / 100)).toFixed(2);
  }

  calculateTableTotal(): string {
    let ret = 0;
    this.documentItemList.forEach((value) => {
      if (value.quantity != undefined) {
        ret += Number(this.calculateTotal(value));
      }
    });
    return ret.toFixed(2);
  }

  deleteItem(index: number) {
    this.documentItemList.splice(index, 1);
  }

  private loadCollapseStatus() {
    this.treatCollapse(window.innerWidth < MOBILE_SIZE ? (): boolean => true : (): boolean => false);
  }

  onCollapse(index: number) {
    this.treatCollapse((i: number): boolean => index == i ? !this.isCollapsed[i] : true);
  }

  private treatCollapse(collapseBehavior: (index: number) => boolean) {
    for (let i: number = 0; i < this.documentItemList.length; i++) {
      this.isCollapsed[i] = collapseBehavior(i);
    }
  }

}
