import {Component, OnInit} from '@angular/core';
import {FlowService} from '../../services/flows/flow.service';
import {Flow} from '../../services/flows/flow';
import {WorkOrderService} from '../../services/work-orders/work-order.service';
import {WorkOrder} from 'src/app/services/work-orders/work-order';
import {PdfReportService} from 'src/app/services/pdf-report/pdf-report.service';
import {ConfirmationDialogService} from 'src/app/services/confirmation-dialog.service';
import {NotificationService} from 'src/app/services/notifications/notification.service';

const CANCELED_FLOW = "CANCELED";
const MOBILE_SIZE = 768;

@Component({
  selector: 'app-work-order-dashboard',
  templateUrl: './work-order-dashboard.component.html',
  styleUrls: ['./work-order-dashboard.component.scss']
})
export class WorkOrderDashboardComponent implements OnInit {

  workOrderFlows: Flow[];
  shownPage: number = 1;
  filteredList: Flow[];
  queryToFilter: string;
  isCollapsed: boolean[] = [];
  pagination: number = 10;

  constructor(
    private flowService: FlowService,
    private workOrderService: WorkOrderService,
    private pdfReportService: PdfReportService,
    private confirmationDialogService: ConfirmationDialogService,
    private notificationService: NotificationService) {

  }

  loadWorkOrderFlows() {
    this.flowService.getWorkOrderFlows()
      .subscribe(flows => {
        this.workOrderFlows = flows;
        this.assignCopy();
      });
  }

  ngOnInit() {
    this.loadWorkOrderFlows();
    this.loadCollapseStatus();
  }

  getTotalPrice(workOrder: WorkOrder): number {
    return this.workOrderService.getTotalPrice(workOrder);
  }

  getPDF(id: number) {
    this.pdfReportService.getPDF(id, this.workOrderService);
  }

  getHTML(id: number) {
    return this.workOrderService.getHTML(id)
      .subscribe(data => console.log(data)),
      error => console.log('Error viewing the file.'),
      () => console.info('OK');
  }

  public openConfirmationDialog(position: number) {
    this.confirmationDialogService.confirm('¿Estas seguro?', '¿De verdad deseas cancelar el flujo?')
      .then((confirmed) => {
        if (confirmed) {
          this.cancel(position);
        }
      })
      .catch(() => console.log('User dismissed the dialog (e.g., by using ESC, clicking the cross icon, or clicking outside the dialog)'));
  }

  cancel(position: number) {
    this.workOrderFlows[position].status = CANCELED_FLOW;
    this.flowService.updateFlow(this.workOrderFlows[position]).subscribe(
      flow => {
        this.loadWorkOrderFlows();
      });
  }

  private loadCollapseStatus() {
    this.treatCollapse(window.innerWidth < MOBILE_SIZE ? (): boolean => true : (): boolean => false);
  }

  onCollapse(index: number) {
    this.treatCollapse((i: number): boolean => index == i ? !this.isCollapsed[i] : true);
  }

  private treatCollapse(collapseBehavior: (index: number) => boolean) {
    for (let i: number = 0; i < this.pagination; i++) {
      this.isCollapsed[i] = collapseBehavior(i);
    }
  }

  assignCopy() {
    this.filteredList = Object.assign([], this.workOrderFlows);
  }

  filterItem() {
    if (!this.queryToFilter) {
      this.assignCopy();
    }

    this.filteredList = Object.assign([], this.workOrderFlows)
      .filter(item => (item.workOrder.vehicle.plate + "" + item.workOrder.vehicle.brand + "" + item.workOrder.vehicle.model).toLowerCase().indexOf(this.queryToFilter.toLowerCase()) > -1)
  }

  sendEmail(id: number) {
    this.notificationService.sendFlow(id, "WORKORDER")
      .subscribe(data => data ? console.log("Email sent") : console.log("Error sending email"));
  }

}
