import { Component, OnInit } from '@angular/core';
import { WorkOrderService } from 'src/app/services/work-orders/work-order.service';
import { ActivatedRoute } from '@angular/router';
import { Location } from '@angular/common';
import { WorkOrder } from 'src/app/services/work-orders/work-order';
import { DocumentItem } from 'src/app/services/document-items/document-item';
import { PdfReportService } from 'src/app/services/pdf-report/pdf-report.service';

const MOBILE_SIZE = 768;

@Component({
  selector: 'app-work-order-preview',
  templateUrl: './work-order-preview.component.html',
  styleUrls: ['./work-order-preview.component.scss']
})
export class WorkOrderPreviewComponent implements OnInit {

  model: WorkOrder;
  isCollapsed: boolean[] = [];

  constructor(
    private workOrderService: WorkOrderService,
    private route: ActivatedRoute,
    private pdfReportService: PdfReportService,
    private location: Location) { }

  ngOnInit() {
    this.loadWorkOrder();
  }

  loadWorkOrder() {
    const id = +this.route.snapshot.paramMap.get('id');
    this.workOrderService.getWorkOrder(id)
      .subscribe(workOrder => {
        this.model = workOrder;
        this.loadCollapseStatus();
      });
  }

  goBack(): void {
    this.location.back();
  }

  calculateTotal(document: DocumentItem) {
    return (document.quantity * (document.productPrice * ((100 + document.taxPercentage) / 100) * (100 - document.discountPercentage) / 100)).toFixed(2);
  }

  calculateTableTotal() {
    let ret = 0;
    this.model.documentItems.forEach((value) => {
      if (value.quantity != undefined) {
        ret += Number(this.calculateTotal(value));
      }
    });
    return ret.toFixed(2);
  }

  getPDF(id: number) {
    this.pdfReportService.getPDF(id, this.workOrderService);
  }

  private loadCollapseStatus() {
    this.treatCollapse(window.innerWidth < MOBILE_SIZE ? (): boolean => true : (): boolean => false);
  }

  onCollapse(index: number) {
    this.treatCollapse((i: number): boolean => index == i ? !this.isCollapsed[i] : true);
  }

  private treatCollapse(collapseBehavior: (index: number) => boolean) {
    for (let i: number = 0; i < this.model.documentItems.length; i++) {
      this.isCollapsed[i] = collapseBehavior(i);
    }
  }

}
